@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Collector.pm,v 1.17 2008/11/23 20:52:00 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -49,6 +49,8 @@ sub initThreads
 }
 
 
+our %collectorTypes;
+
 ## One collector module instance holds all leaf tokens which
 ## must be collected at the same time.
 
@@ -62,14 +64,25 @@ sub new
         $options{'-Name'} = "Collector";
     }
 
+    # Repeat so many cycles immediately at start
+    if( $Torrus::Collector::fastCycles > 0 )
+    {
+        $options{'-FastCycles'} = $Torrus::Collector::fastCycles;
+    }
+    
     my $class = ref($proto) || $proto;
     my $self  = $class->SUPER::new( %options );
     bless $self, $class;
 
-    foreach my $collector_type ( keys %Torrus::Collector::collectorTypes )
+    $self->{'types_sorted'} = [];
+    
+    foreach my $collector_type
+        ( sort {$collectorTypes{$a} <=> $collectorTypes{$b}}
+          keys %collectorTypes )
     {
         $self->{'types'}{$collector_type} = {};
         $self->{'types_in_use'}{$collector_type} = 0;
+        push(@{$self->{'types_sorted'}}, $collector_type);
     }
 
     foreach my $storage_type ( keys %Torrus::Collector::storageTypes )
@@ -100,7 +113,7 @@ sub addTarget
     $self->{'targets'}{$token}{'path'} = $config_tree->path($token);
 
     my $collector_type = $config_tree->getNodeParam($token, 'collector-type');
-    if( not $Torrus::Collector::collectorTypes{$collector_type} )
+    if( not $collectorTypes{$collector_type} )
     {
         Error('Unknown collector type: ' . $collector_type);
         return;
@@ -397,8 +410,8 @@ sub run
     my $self = shift;
 
     undef $self->{'values'};
-
-    while( my ($collector_type, $ref) = each %{$self->{'types'}} )
+    
+    foreach my $collector_type ( @{$self->{'types_sorted'}} )
     {
         next unless $self->{'types_in_use'}{$collector_type};
 
@@ -412,7 +425,8 @@ sub run
                                         -Wait => 1 );
         }
         
-        &{$Torrus::Collector::runCollector{$collector_type}}( $self, $ref );
+        &{$Torrus::Collector::runCollector{$collector_type}}
+        ( $self, $self->collectorData($collector_type) );
         
         if( defined( $self->{'config_tree'} ) )
         {
@@ -606,14 +620,26 @@ sub beforeRun
     
     if( $actual_ts >= $known_ts or not $data->{'targets_initialized'} )
     {
+        my $db_lock;
+        my $cursor_lock;
+        if( $Torrus::Collector::exclusiveStartupLock )
+        {
+            Info('Acquiring an exclusive lock for configuration slurp');
+            $db_lock =
+                new Torrus::DB( 'collector_lock', -WriteAccess => 1 );        
+            $cursor_lock = $db_lock->cursor( -Write => 1 );
+        }
+        
         Info('Initializing tasks for collector instance ' . $instance);
         Debug("Config TS: $actual_ts, Collector TS: $known_ts");
         my $init_start = time();
 
         my $targets = {};
 
-        my $db_tokens = new Torrus::DB('collector_tokens' . '_' . $instance,
-                                       -Subdir => $tree);
+        my $db_tokens =
+            new Torrus::DB('collector_tokens' . '_' . $instance . '_' .
+                           $config_tree->{'ds_config_instance'},
+                           -Subdir => $tree);
         
         my $cursor = $db_tokens->cursor();
         while( my ($token, $schedule) = $db_tokens->next($cursor) )
@@ -627,10 +653,11 @@ sub beforeRun
 
             &Torrus::DB::checkInterrupted();
         }
+        $db_tokens->c_close($cursor);
         undef $cursor;
         $db_tokens->closeNow();
         undef $db_tokens;
-
+        
         &Torrus::DB::checkInterrupted();
 
         # Set the timestamp
@@ -657,10 +684,33 @@ sub beforeRun
                 $self->addTask( $collector );
             }
         }
+
+        if( $Torrus::Collector::exclusiveStartupLock )
+        {
+            $db_lock->c_close($cursor_lock);
+            undef $cursor_lock;
+            $db_lock->closeNow();
+            undef $db_lock;
+        }
+
         Verbose(sprintf("Tasks initialization finished in %d seconds",
                         time() - $init_start));
 
-        $data->{'targets_initialized'} = 1;
+        $data->{'targets_initialized'} = 1;        
+        Info('Tasks for collector instance ' . $instance . ' initialized');
+
+        foreach my $collector_type ( keys %Torrus::Collector::collectorTypes )
+        {
+            if( ref($Torrus::Collector::initCollectorGlobals{
+                $collector_type}) )
+            {
+                &{$Torrus::Collector::initCollectorGlobals{
+                    $collector_type}}($tree, $instance);
+                
+                Verbose('Initialized collector globals for type: ' .
+                        $collector_type);
+            }
+        }
     }
     
     Torrus::TimeStamp::release();
