@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: ExternalStorage.pm,v 1.8 2008/10/28 11:13:50 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::ExternalStorage;
@@ -196,15 +196,22 @@ sub processCounter
     my $timestamp = shift;
 
     my $sref = $collector->storageData( 'ext' );
-    my $ret;
 
     if( isDebug() )
     {
         Debug('ExternalStorage::processCounter: token=' . $token .
               ' value=' . $value . ' timestamp=' . $timestamp);
     }
+
+    if( $value eq 'U' )
+    {
+        # the agent rebooted, so we flush the counter
+        delete $sref->{'prevCounter'}{$token};
+        return undef;
+    }
         
     $value = Math::BigInt->new( $value );
+    my $ret;
     
     if( exists( $sref->{'prevCounter'}{$token} ) )
     {
