@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RRDStorage.pm,v 1.19 2008/11/23 20:52:00 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::RRDStorage;
@@ -385,10 +385,16 @@ sub updateRRD
 
     if( $ds_conflict and $moveConflictRRD )
     {
+        if( not -f $filename )
+        {
+            Error($filename . 'is not a regular file');
+            return;
+        }
+        
         my( $sec, $min, $hour, $mday, $mon, $year) = localtime( time() );
         my $destfile = sprintf('%s_%04d%02d%02d%02d%02d',
                                $filename,
-                               $year + 1900, $mon, $mday, $hour, $min);
+                               $year + 1900, $mon+1, $mday, $hour, $min);
         
         my $destdir = $conflictRRDPath;
         if( defined( $destdir ) and -d $destdir )
@@ -397,6 +403,7 @@ sub updateRRD
             my $fname = pop( @fpath );
             $destfile = $destdir . '/' . $fname;
         }
+
         Warn('Moving the conflicted RRD file ' . $filename .
              ' to ' . $destfile);
         rename( $filename, $destfile ) or
@@ -426,6 +433,8 @@ sub updateRRD
 
     my $step = $collector->period();
 
+    my $has_values = 0;
+    
     foreach my $ds ( keys %ds_updating )
     {
         my $token = $ds_updating{$ds};
@@ -440,6 +449,7 @@ sub updateRRD
         if( ref $sref->{'values'}{$token} )
         {
             ($value, $timestamp, $uptime) = @{$sref->{'values'}{$token}};
+            $has_values = 1;
         }
 
         push( @timestamps, $timestamp );
@@ -456,6 +466,11 @@ sub updateRRD
         $values .= ':'. $value;
     }
 
+    if( not $has_values )
+    {
+        return;
+    }
+    
     # Get the average timestamp
     my $sum = 0;
     map {$sum += $_} @timestamps;
