@@ -1,4 +1,4 @@
-#  Copyright (C) 2002-2007  Stanislav Sinyagin
+#  Copyright (C) 2002-2011  Stanislav Sinyagin
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,6 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: SNMP.pm,v 1.49 2008/11/19 21:23:35 ssinyagin Exp $
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Collector::SNMP;
@@ -22,6 +21,7 @@ package Torrus::Collector::SNMP;
 use Torrus::Collector::SNMP_Params;
 use Torrus::ConfigTree;
 use Torrus::Log;
+use Torrus::SNMP_Failures;
 
 use strict;
 use Net::hostent;
@@ -29,9 +29,11 @@ use Socket;
 use Net::SNMP qw(:snmp);
 use Math::BigInt;
 
+
 # Register the collector type
 $Torrus::Collector::collectorTypes{'snmp'} = 1;
 
+
 # List of needed parameters and default values
 
 $Torrus::Collector::params{'snmp'} = {
@@ -70,12 +72,18 @@ my %snmpV1Hosts;
 # SNMP tables lookup maps
 my %maps;
 
+# Old lookup maps, used temporarily during refresh cycle
+my %oldMaps;
+
 # How frequent we refresh the SNMP mapping
 our $mapsRefreshPeriod;
 
 # Random factor in refresh period
 our $mapsRefreshRandom;
 
+# Time period after configuration re-compile when we refresh existing mappings
+our $mapsUpdateInterval;
+
 # how often we check for expired maps
 our $mapsExpireCheckPeriod;
 
@@ -99,6 +107,48 @@ my %hostUnreachableRetry;
 my %unreachableHostDeleted;
 
 
+our $db_failures;
+
+# Flush stats after a restart or recompile
+$Torrus::Collector::initCollectorGlobals{'snmp'} =
+    \&Torrus::Collector::SNMP::initCollectorGlobals;
+
+sub initCollectorGlobals
+{
+    my $tree = shift;
+    my $instance = shift;
+    
+    if( not defined( $db_failures ) )
+    {
+        $db_failures =
+            new Torrus::SNMP_Failures( -Tree => $tree,
+                                       -Instance => $instance,
+                                       -WriteAccess => 1 );
+    }
+
+    if( defined( $db_failures ) )
+    {
+        $db_failures->init();
+    }
+
+    # re-init counters and collect garbage
+    %oldMaps = ();
+    %hostUnreachableSeen = ();
+    %hostUnreachableRetry = ();
+    %unreachableHostDeleted = ();
+    
+    # Configuration re-compile was probably caused by new object instances
+    # appearing on the monitored devices. Here we force the maps to refresh
+    # soon enough in order to catch up with the changes
+
+    my $now = time();    
+    foreach my $maphash ( keys %mapsExpire )
+    {
+        $mapsExpire{$maphash} = int( $now + rand( $mapsUpdateInterval ) );
+    }    
+}
+
+
 # This is first executed per target
 
 $Torrus::Collector::initTarget{'snmp'} = \&Torrus::Collector::SNMP::initTarget;
@@ -111,18 +161,17 @@ sub initTarget
     my $token = shift;
 
     my $tref = $collector->tokenData( $token );
-    my $cref = $collector->collectorData( 'snmp' );
 
     $collector->registerDeleteCallback
         ( $token, \&Torrus::Collector::SNMP::deleteTarget );
 
-    my $hostname = getHostname( $collector, $token );
-    if( not defined( $hostname ) )
+    my $hosthash = getHostHash( $collector, $token );
+    if( not defined( $hosthash ) )
     {
         return 0;
     }
 
-    $tref->{'hostname'} = $hostname;
+    $tref->{'hosthash'} = $hosthash;
     
     return Torrus::Collector::SNMP::initTargetAttributes( $collector, $token );
 }
@@ -138,25 +187,9 @@ sub initTargetAttributes
     my $tref = $collector->tokenData( $token );
     my $cref = $collector->collectorData( 'snmp' );
 
-    my $hostname = $tref->{'hostname'};
-    my $port = $collector->param($token, 'snmp-port');
+    my $hosthash = $tref->{'hosthash'};
+
     my $version = $collector->param($token, 'snmp-version');
-
-    my $community;
-    if( $version eq '1' or $version eq '2c' )
-    {
-        $community = $collector->param($token, 'snmp-community');
-    }
-    else
-    {
-        # We use community string to identify the agent.
-        # For SNMPv3, it's the user name
-        $community = $collector->param($token, 'snmp-username');
-    }
-
-    my $hosthash = join('|', $hostname, $port, $community);
-    $tref->{'hosthash'} = $hosthash;
-
     if( $version eq '1' )
     {
         $snmpV1Hosts{$hosthash} = 1;
@@ -229,13 +262,11 @@ sub initTargetAttributes
 }
 
 
-sub getHostname
+sub getHostHash
 {
     my $collector = shift;
     my $token = shift;
 
-    my $cref = $collector->collectorData( 'snmp' );
-
     my $hostname = $collector->param($token, 'snmp-host');
     my $domain = $collector->param($token, 'domain-name');
     
@@ -246,7 +277,22 @@ sub getHostname
         $hostname .= '.' . $domain;
     }
     
-    return $hostname;
+    my $port = $collector->param($token, 'snmp-port');
+    my $version = $collector->param($token, 'snmp-version');
+
+    my $community;
+    if( $version eq '1' or $version eq '2c' )
+    {
+        $community = $collector->param($token, 'snmp-community');
+    }
+    else
+    {
+        # We use community string to identify the agent.
+        # For SNMPv3, it's the user name
+        $community = $collector->param($token, 'snmp-username');
+    }
+
+    return join('|', $hostname, $port, $community);
 }
 
 
@@ -350,8 +396,10 @@ sub openNonblockingSession
     if( not defined($session) )
     {
         Error('Cannot create SNMP session for ' . $hosthash . ': ' . $error);
+        return undef;
     }
-    else
+    
+    if( $collector->param($token, 'snmp-transport') eq 'udp' )
     {
         # We set SO_RCVBUF only once, because Net::SNMP shares
         # one UDP socket for all sessions.
@@ -375,12 +423,13 @@ sub openNonblockingSession
                 Debug('Set SO_RCVBUF to ' . $buflen);
             }
         }
+    }
 
-        my $maxmsgsize = $collector->param($token, 'snmp-max-msg-size');
-        if( defined( $maxmsgsize ) and $maxmsgsize > 0 )
-        {
-            $session->max_msg_size( $maxmsgsize );
-        }
+    my $maxmsgsize = $collector->param($token, 'snmp-max-msg-size');
+    if( defined( $maxmsgsize ) and $maxmsgsize > 0 )
+    {
+        $session->max_msg_size( $maxmsgsize );
+        
     }
     
     return $session;
@@ -465,7 +514,7 @@ sub expandOidMappings
 
             my $result = $session->get_request( -varbindlist => [$key] );
             $session->close();
-            if( defined $result )
+            if( defined $result and defined($result->{$key}) )
             {
                 $value = $result->{$key};
                 $cref->{'value-lookups'}{$hosthash}{$key} = $value;
@@ -493,7 +542,7 @@ sub expandOidMappings
         }
     }
 
-    # Debug("OID expanded: $oid_in -> $oid");
+    # Debug('OID expanded: ' . $oid_in . ' -> ' . $oid');
     return $oid;
 }
 
@@ -509,16 +558,26 @@ sub lookupMap
 
     my $cref = $collector->collectorData( 'snmp' );
     my $maphash = join('#', $hosthash, $map);
-
+    
     if( not defined( $maps{$hosthash}{$map} ) )
-    {        
+    {
+        my $ret;
+
+        if( defined( $oldMaps{$hosthash}{$map} ) and
+            defined( $key ) )
+        {
+            $ret = $oldMaps{$hosthash}{$map}{$key};
+        }
+        
         if( $mapLookupScheduled{$maphash} )
         {
-            return undef;
+            return $ret;
         }
-
-        if( scalar(@mappingSessions) >=
-            $Torrus::Collector::SNMP::maxSessionsPerDispatcher )
+        
+        if( not $collector->didNotRun()
+            and
+            (scalar(@mappingSessions) >=
+             $Torrus::Collector::SNMP::maxSessionsPerDispatcher) )
         {
             snmp_dispatcher();
             @mappingSessions = ();
@@ -526,12 +585,12 @@ sub lookupMap
         }
 
         # Retrieve map from host
-        Debug("Retrieving map $map from $hosthash");
+        Debug('Retrieving map ' . $map . ' from ' . $hosthash);
 
         my $session = openNonblockingSession( $collector, $token, $hosthash );
         if( not defined($session) )
         {
-            return undef;
+            return $ret;
         }
         else
         {
@@ -553,7 +612,7 @@ sub lookupMap
                      rand( $mapsRefreshPeriod * $mapsRefreshRandom ) );
         }
         
-        return undef;
+        return $ret;
     }
 
     if( defined( $key ) )
@@ -659,6 +718,13 @@ sub probablyDead
     else
     {
         $hostUnreachableSeen{$hosthash} = time();
+
+        if( defined( $db_failures ) )
+        {
+            $db_failures->host_failure('unreachable', $hosthash);
+            $db_failures->set_counter('unreachable',
+                                      scalar( keys %hostUnreachableSeen));
+        }
     }
 
     if( $probablyAlive )
@@ -690,6 +756,15 @@ sub probablyDead
         delete $hostUnreachableSeen{$hosthash};
         delete $hostUnreachableRetry{$hosthash};
         $unreachableHostDeleted{$hosthash} = 1;
+
+        if( defined( $db_failures ) )
+        {
+            $db_failures->host_failure('deleted', $hosthash);
+            $db_failures->set_counter('unreachable',
+                                      scalar( keys %hostUnreachableSeen));
+            $db_failures->set_counter('deleted',
+                                      scalar( keys %unreachableHostDeleted));
+        }
     }
     
     return $probablyAlive;
@@ -702,8 +777,6 @@ sub checkUnreachableRetry
     my $collector = shift;
     my $hosthash = shift;
 
-    my $cref = $collector->collectorData( 'snmp' );
-
     my $ret = 1;
     if( $hostUnreachableSeen{$hosthash} )
     {
@@ -734,7 +807,6 @@ sub isHostDead
     my $collector = shift;
     my $hosthash = shift;
 
-    my $cref = $collector->collectorData( 'snmp' );
     return $unreachableHostDeleted{$hosthash};
 }
 
@@ -744,10 +816,15 @@ sub hostReachableAgain
     my $collector = shift;
     my $hosthash = shift;
     
-    my $cref = $collector->collectorData( 'snmp' );
     if( exists( $hostUnreachableSeen{$hosthash} ) )
     {
         delete $hostUnreachableSeen{$hosthash};
+        if( defined( $db_failures ) )
+        {
+            $db_failures->remove_host($hosthash);            
+            $db_failures->set_counter('unreachable',
+                                      scalar( keys %hostUnreachableSeen));
+        }
     }
 }
 
@@ -854,7 +931,7 @@ sub runCollector
             
             if( not defined($session) )
             {
-                return 0;
+                next;
             }
             else
             {
@@ -884,7 +961,7 @@ sub runCollector
                     
                     if( Torrus::Log::isDebug() )
                     {
-                        Debug("Sending SNMP PDU to $hosthash:");
+                        Debug('Sending SNMP PDU to ' . $hosthash . ':');
                         foreach my $oid ( @pdu_oids )
                         {
                             Debug($oid);
@@ -986,7 +1063,7 @@ sub callback
         if( defined $uptimeTicks )
         {
             $uptime = $uptimeTicks / 100;
-            Debug("Uptime: $uptime");
+            Debug('Uptime: ' . $uptime);
         }
         else
         {
@@ -1035,6 +1112,12 @@ sub callback
                     
                     foreach my $token ( keys %{$pdu_tokens->{$oid}} )
                     {
+                        if( defined( $db_failures ) )
+                        {
+                            $db_failures->mib_error
+                                ($hosthash, $collector->path($token));
+                        }
+
                         $collector->deleteTarget($token);
                     }
                 }
@@ -1129,6 +1212,8 @@ sub postProcess
                     }
                     my $reptoken = $tokens[0];
 
+                    # save the map for the time of refresh                    
+                    $oldMaps{$hosthash}{$map} = $maps{$hosthash}{$map};
                     delete $maps{$hosthash}{$map};
 
                     # this will schedule the map retrieval for the next
@@ -1159,6 +1244,79 @@ sub postProcess
     }    
 }
 
+
+####  ====  SNMP Reachability Collector ====
+
+# Register the collector type (it should always run after snmp collector)
+$Torrus::Collector::collectorTypes{'snmp-reachable'} = 2;
+
+
+# List of needed parameters and default values
+
+$Torrus::Collector::params{'snmp-reachable'} = {
+    'snmp-version'      => undef,
+    'snmp-port'         => undef,
+    'snmp-community'    => undef,
+    'snmp-username'     => undef,
+    'domain-name'       => undef,
+    'snmp-host'         => undef,
+};
+
+
+# This is first executed per target
+
+$Torrus::Collector::initTarget{'snmp-reachable'} = \&reachable_initTarget;
+
+
+sub reachable_initTarget
+{
+    my $collector = shift;
+    my $token = shift;
+        
+    my $cref = $collector->collectorData('snmp-reachable');
+    my $tref = $collector->tokenData( $token );    
+    my $hosthash = getHostHash( $collector, $token );
+    if( not defined( $hosthash ) )
+    {
+        return 0;
+    }
+
+    $tref->{'hosthash'} = $hosthash;
+    $cref->{'targets'}{$hosthash}{$token} = 1;
+
+    return 1;
+}
+
+
+# Main collector cycle
+
+$Torrus::Collector::runCollector{'snmp-reachable'} = \&reachable_runCollector;
+
+sub reachable_runCollector
+{
+    my $collector = shift;
+    my $cref = shift;
+
+    if( not defined( $db_failures ) )
+    {
+        Error('$db_failures is not initialized. ' .
+              'snmp-reachable collector cannot continue');
+        return;
+    }
+
+    my $timestamp = time();
+
+    while(my ($hosthash, $r) = each %{$cref->{'targets'}} )
+    {
+        foreach my $token (keys %{$r})
+        {
+            my $val = $db_failures->is_host_available($hosthash) ? 100:0;
+            $collector->setValue( $token, $val, $timestamp, 0 );
+        }
+    }
+}
+
+        
 1;
 
 
