@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: ConfigTree.pm,v 1.8 2007/04/11 15:09:17 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -43,7 +43,11 @@ sub new
     defined( $self->{'db_config_instances'} ) or return( undef );
 
     my $i = $self->{'db_config_instances'}->get('ds:' . $self->{'treename'});
-    $i = 0 unless defined( $i );
+    if( not defined($i) )
+    {
+        $i = 0;
+        $self->{'first_time_created'} = 1;
+    }
 
     my $dsConfInstance = sprintf( '%d', $i );
 
@@ -82,7 +86,7 @@ sub new
             {
                 $self->{'db_config_instances'}->c_put( $cursor, $key, 1 );
             }
-            undef $cursor;
+            $self->{'db_config_instances'}->c_close($cursor);
             if( not $ok )
             {
                 return undef;
@@ -209,6 +213,7 @@ sub new
             my( $param, $prop ) = split( /:/o, $key );
             $self->{'paramprop'}{$prop}{$param} = $val;
         }
+        $self->{'db_paramprops'}->c_close($cursor);
         undef $cursor;
         $self->{'db_paramprops'}->closeNow();
         delete $self->{'db_paramprops'};
@@ -216,11 +221,12 @@ sub new
 
     
     $self->{'db_sets'} =
-        new Torrus::DB('tokensets',
+        new Torrus::DB('tokensets_' . $dsConfInstance,
                        -Subdir => $self->{'treename'}, -Btree => 0,
                        -WriteAccess => 1, -Truncate => $options{'-Rebuild'});
     defined( $self->{'db_sets'} ) or return( undef );
 
+
     $self->{'db_nodepcache'} =
         new Torrus::DB('nodepcache_' . $dsConfInstance,
                        -Subdir => $self->{'treename'}, -Btree => 1,
@@ -229,6 +235,15 @@ sub new
                                      not $options{'-NoDSRebuild'}));
     defined( $self->{'db_nodepcache'} ) or return( undef );
 
+
+    $self->{'db_nodeid'} =
+        new Torrus::DB('nodeid_' . $dsConfInstance,
+                       -Subdir => $self->{'treename'}, -Btree => 1,
+                       -WriteAccess => 1,
+                       -Truncate => ($options{'-Rebuild'} and
+                                     not $options{'-NoDSRebuild'}));
+    defined( $self->{'db_nodeid'} ) or return( undef );
+
     return $self;
 }
 
@@ -245,7 +260,7 @@ sub DESTROY
         my $cursor = $self->{'db_config_instances'}->cursor( -Write => 1 );
         $self->{'db_config_instances'}->c_put
             ( $cursor, 'compiling:' . $self->{'treename'}, 0 );
-        undef $cursor;
+        $self->{'db_config_instances'}->c_close($cursor);
     }
     else
     {
@@ -325,7 +340,7 @@ sub waitReaders
                     'timestamp' => $timestamp } );
             }
         }
-        undef $cursor;
+        $self->{'db_readers'}->c_close($cursor);
         if( @readers > 0 )
         {
             Info('Waiting for ' . scalar(@readers) . ' readers:');
@@ -366,7 +381,7 @@ sub waitReaders
                         $self->{'db_readers'}->c_del( $cursor );
                     }
                 }
-                undef $cursor;
+                $self->{'db_readers'}->c_close($cursor);
                 $noReaders = 1;
             }
         }
@@ -463,20 +478,41 @@ sub nodeExists
     return defined( $self->{'db_dsconfig'}->get( 'pt:'.$path ) );
 }
 
+
+sub nodeType
+{
+    my $self = shift;
+    my $token = shift;
+
+    my $type = $self->{'nodetype_cache'}{$token};
+    if( not defined( $type ) )
+    {
+        $type = $self->{'db_dsconfig'}->get( 'n:'.$token );
+        if( not defined( $type ) )
+        {
+            $type = -1;
+        }
+        $self->{'nodetype_cache'}{$token} = $type;
+    }
+    return $type;
+}
+    
+
 sub isLeaf
 {
     my $self = shift;
     my $token = shift;
 
-    return( $self->{'db_dsconfig'}->get( 'n:'.$token ) == 1 );
+    return ( $self->nodeType($token) == 1 );
 }
 
+
 sub isSubtree
 {
     my $self = shift;
     my $token = shift;
 
-    return( $self->{'db_dsconfig'}->get( 'n:'.$token ) == 0 );
+    return( $self->nodeType($token) == 0 );
 }
 
 # Returns the real token or undef
@@ -485,7 +521,7 @@ sub isAlias
     my $self = shift;
     my $token = shift;
 
-    return( ( $self->{'db_dsconfig'}->get( 'n:'.$token ) == 2 ) ?
+    return( ( $self->nodeType($token) == 2 ) ?
             $self->{'db_dsconfig'}->get( 'a:'.$token ) : undef );
 }
 
@@ -495,7 +531,7 @@ sub getAliases
     my $self = shift;
     my $token = shift;
 
-    return split( /,/o, $self->{'db_dsconfig'}->get( 'ar:'.$token ) );
+    return $self->{'db_dsconfig'}->getListItems('ar:'.$token);
 }
 
 
@@ -525,17 +561,36 @@ sub retrieveNodeParam
     my $token = shift;
     my $param = shift;
 
-    my $value;
+    # walk up the tree and save the grandparent's value at parent's cache
+    
+    my $value;    
     my $currtoken = $token;
+    my @ancestors;
+    my $walked = 0;
+    
     while( not defined($value) and defined($currtoken) )
     {
         $value = $self->getParam( $currtoken, $param, 1 );
         if( not defined $value )
         {
-            # get the parent's token
+            if( $walked )
+            {
+                push( @ancestors, $currtoken );
+            }
+            else
+            {
+                $walked = 1;
+            }
+            # walk up to the parent
             $currtoken = $self->getParent($currtoken);
         }
     }
+
+    foreach my $ancestor ( @ancestors )
+    {
+        $self->{'paramcache'}{$ancestor}{$param} = $value;
+    }
+    
     return $self->expandNodeParam( $token, $param, $value );
 }
 
@@ -572,7 +627,7 @@ sub expandSubstitutions
         $changed = 0;
 
         # Substitute definitions
-        if( index($value, "\$") >= 0 )
+        if( index($value, '$') >= 0 )
         {
             if( not $value =~ /\$(\w+)/o )
             {
@@ -685,6 +740,7 @@ sub getNodeParam
     return $value;
 }
 
+
 sub getParamNames
 {
     my $self = shift;
@@ -693,29 +749,22 @@ sub getParamNames
 
     my $db = $fromDS ? $self->{'db_dsconfig'} : $self->{'db_otherconfig'};
 
-    my $list = $db->get( 'Pl:'.$name );
-    if( defined( $list ) )
-    {
-        return split( /,/o, $list );
-    }
-    else
-    {
-        return ();
-    }
+    return $db->getListItems('Pl:'.$name);
 }
 
+
 sub getParams
 {
     my $self = shift;
     my $name = shift;
     my $fromDS = shift;
 
-    my %ret = ();
+    my $ret = {};
     foreach my $param ( $self->getParamNames( $name, $fromDS ) )
     {
-        $ret{$param} = $self->getParam( $name, $param, $fromDS );
+        $ret->{$param} = $self->getParam( $name, $param, $fromDS );
     }
-    return \%ret;
+    return $ret;
 }
 
 sub getParent
@@ -746,18 +795,7 @@ sub getChildren
     }
     else
     {
-        my $children = $self->{'db_dsconfig'}->get( 'c:'.$token );
-
-        if( defined $children )
-        {
-            my @clist = split(/,/o, $children);
-            map { $self->{'parentcache'}{$_} = $token; } @clist;
-            return @clist;
-        }
-        else
-        {
-            return ();
-        }
+        return $self->{'db_dsconfig'}->getListItems( 'c:'.$token );
     }
 }
 
@@ -888,6 +926,37 @@ sub getRelative
 }
 
 
+sub getNodeByNodeid
+{
+    my $self = shift;
+    my $nodeid = shift;
+
+    return $self->{'db_nodeid'}->get( $nodeid );
+}
+
+# Returns arrayref or undef.
+# Each element is an arrayref to [nodeid, token] pair
+sub searchNodeidPrefix
+{
+    my $self = shift;
+    my $prefix = shift;
+
+    return $self->{'db_nodeid'}->searchPrefix( $prefix );
+}
+
+
+# Returns arrayref or undef.
+# Each element is an arrayref to [nodeid, token] pair
+sub searchNodeidSubstring
+{
+    my $self = shift;
+    my $substring = shift;
+
+    return $self->{'db_nodeid'}->searchSubstring( $substring );
+}
+
+
+
 sub getDefaultView
 {
     my $self = shift;
@@ -948,9 +1017,7 @@ sub getInstanceParam
 sub getViewNames
 {
     my $self = shift;
-    my $vlist = $self->{'db_otherconfig'}->get( 'V:' );
-
-    return defined($vlist) ? split(/,/o, $vlist) : ();
+    return $self->{'db_otherconfig'}->getListItems( 'V:' );
 }
 
 
@@ -965,9 +1032,7 @@ sub viewExists
 sub getMonitorNames
 {
     my $self = shift;
-    my $mlist = $self->{'db_otherconfig'}->get( 'M:' );
-
-    return defined($mlist) ? split(/,/o, $mlist) : ();
+    return $self->{'db_otherconfig'}->getListItems( 'M:' );
 }
 
 sub monitorExists
@@ -981,9 +1046,7 @@ sub monitorExists
 sub getActionNames
 {
     my $self = shift;
-    my $alist = $self->{'db_otherconfig'}->get( 'A:' );
-
-    return defined($alist) ? split(/,/o, $alist) : ();
+    return $self->{'db_otherconfig'}->getListItems( 'A:' );
 }
 
 
@@ -1032,8 +1095,7 @@ sub tsetExists
 sub getTsets
 {
     my $self = shift;
-    my $list = $self->{'db_sets'}->get('S:');
-    return defined($list) ? split(/,/o, $list) : ();
+    return $self->{'db_sets'}->getListItems('S:');
 }
 
 sub tsetMembers
@@ -1041,8 +1103,16 @@ sub tsetMembers
     my $self = shift;
     my $tset = shift;
 
-    my $list = $self->{'db_sets'}->get('s:'.$tset);
-    return defined($list) ? split(/,/o, $list) : ();
+    return $self->{'db_sets'}->getListItems('s:'.$tset);
+}
+
+sub tsetMemberOrigin
+{
+    my $self = shift;
+    my $tset = shift;
+    my $token = shift;
+    
+    return $self->{'db_sets'}->get('o:'.$tset.':'.$token);
 }
 
 sub tsetAddMember
@@ -1050,8 +1120,10 @@ sub tsetAddMember
     my $self = shift;
     my $tset = shift;
     my $token = shift;
+    my $origin = shift;
 
     $self->{'db_sets'}->addToList('s:'.$tset, $token);
+    $self->{'db_sets'}->put('o:'.$tset.':'.$token, $origin);
 }
 
 
@@ -1062,6 +1134,7 @@ sub tsetDelMember
     my $token = shift;
 
     $self->{'db_sets'}->delFromList('s:'.$tset, $token);
+    $self->{'db_sets'}->del('o:'.$tset.':'.$token);
 }
 
 # Definitions manipulation
@@ -1076,9 +1149,7 @@ sub getDefinition
 sub getDefinitionNames
 {
     my $self = shift;
-    my $dlist = $self->{'db_dsconfig'}->get( 'D:' );
-
-    return defined($dlist) ? split(/,/o, $dlist) : ();
+    return $self->{'db_dsconfig'}->getListItems( 'D:' );
 }
 
 
