@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Validator.pm,v 1.12 2008/09/25 21:24:46 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -128,7 +128,8 @@ my %view_params =
                      'rrprint' => {'start' => undef,
                                    'print-cf' => undef},
                      'html' => {'html-template' => undef},
-                     'adminfo' => undef}
+                     'adminfo' => undef,
+                     'rpc' => undef}
      );
 
 
