@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Writer.pm,v 1.19 2008/09/25 21:24:47 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 #
@@ -72,12 +72,16 @@ sub new
     foreach my $instance ( 0 .. ($self->{'collectorInstances'} - 1) )
     {
         $self->{'db_collectortokens'}->[$instance] =
-            new Torrus::DB( 'collector_tokens' . '_' . $instance,
+            new Torrus::DB( 'collector_tokens' . '_' .
+                            $instance . '_' . $self->{'ds_config_instance'},
                             -Subdir => $self->treeName(),
                             -WriteAccess => 1,
                             -Truncate    => 1 );
     }
-        
+
+    # delay writing of frequently changed values
+    $self->{'db_dsconfig'}->delay();
+    $self->{'db_otherconfig'}->delay();    
     return $self;
 }
 
@@ -148,6 +152,7 @@ sub initRoot
         $self->{'db_dsconfig'}->put( 'pt:/', $token );
         $self->{'db_dsconfig'}->put( 'tp:'.$token, '/' );
         $self->{'db_dsconfig'}->put( 'n:'.$token, 0 );
+        $self->{'nodetype_cache'}{$token} = 0;
     }
 }
 
@@ -195,6 +200,7 @@ sub addChild
             $nodeType = 1; # leaf
         }
         $self->{'db_dsconfig'}->put( 'n:'.$ctoken, $nodeType );
+        $self->{'nodetype_cache'}{$ctoken} = $nodeType;
     }
     return $ctoken;
 }
@@ -347,6 +353,10 @@ sub finalize
 
     if( $status )
     {
+        # write delayed data
+        $self->{'db_dsconfig'}->commit();
+        $self->{'db_otherconfig'}->commit();    
+        
         Verbose('Configuration has compiled successfully. Switching over to ' .
              'DS config instance ' . $self->{'ds_config_instance'} .
              ' and Other config instance ' .
@@ -388,6 +398,8 @@ sub postProcess
     return $ok;
 }
 
+
+
 sub postProcessNodes
 {
     my $self = shift;
@@ -402,6 +414,26 @@ sub postProcessNodes
         $token = $self->token('/');
     }
 
+    my $nodeid = $self->getNodeParam( $token, 'nodeid', 1 );
+    if( defined( $nodeid ) )
+    {
+        # verify the uniqueness of nodeid
+        
+        my $oldToken = $self->{'db_nodeid'}->get($nodeid);
+        if( defined($oldToken) )
+        {
+            Error('Non-unique nodeid ' . $nodeid .
+                  ' in ' . $self->path($token) .
+                  ' and ' . $self->path($oldToken));
+            $ok = 0;
+        }
+        else
+        {
+            $self->{'db_nodeid'}->put($nodeid, $token);
+        }
+    }
+
+    
     if( $self->isLeaf($token) )
     {
         # Process static tokenset members
@@ -420,7 +452,7 @@ sub postProcessNodes
                 }
                 else
                 {
-                    $self->tsetAddMember( $tsetName, $token );
+                    $self->tsetAddMember( $tsetName, $token, 'static' );
                 }
             }
         }
