@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: XMLCompiler.pm,v 1.8 2008/09/25 21:24:47 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 
@@ -241,19 +241,8 @@ sub compile_params
         else
         {
             # Remove spaces in the head and tail.
-            $value =~ s/^\s+//o;
-            $value =~ s/\s+$//o;
-
-            if( $param eq 'legend' )
-            {
-                # Remove space around delimiters
-                $value =~ s/\s*\:\s*/\:/go;
-                $value =~ s/\s*\;\s*/\;/go;
-            }
-            elsif( $param eq 'ds-type' and $value eq 'RRDfile' )
-            {
-                $value = 'rrd-file';
-            }
+            $value =~ s/^\s+//om;
+            $value =~ s/\s+$//om;
 
             if( $isDS )
             {
@@ -303,19 +292,6 @@ sub compile_ds
 
 
 
-sub expand_name
-{
-    my $self = shift;
-    my $parentpath = shift;
-    my $childname = shift;
-    if( index($childname, '$PARENT') > -1 )
-    {
-        my @pathparts = split(/\//o, $parentpath);
-        my $parentname = pop @pathparts;
-        $childname =~ s/\$PARENT/$parentname/g;
-    }
-    return $childname;
-}
 
 sub validate_nodename
 {
@@ -331,11 +307,10 @@ sub compile_subtrees
     my $self = shift;
     my $node = shift;
     my $token = shift;
+    my $iamLeaf = shift;
+    
     my $ok = 1;
 
-    my $path = $self->path($token);
-    my $iamLeaf = $node->nodeName() eq 'leaf';
-
     # Apply templates
 
     foreach my $templateapp ( $node->getChildrenByTagName('apply-template') )
@@ -343,6 +318,7 @@ sub compile_subtrees
         my $name = $templateapp->getAttribute('name');
         if( not $name )
         {
+            my $path = $self->path($token);
             Error("Template application without a name at $path"); $ok = 0;
         }
         else
@@ -350,11 +326,13 @@ sub compile_subtrees
             my $template = $self->{'Templates'}->{$name};
             if( not defined $template )
             {
+                my $path = $self->path($token);
                 Error("Cannot find template named $name at $path"); $ok = 0;
             }
             else
             {
-                $ok = $self->compile_subtrees( $template, $token ) ? $ok:0;
+                $ok = $self->compile_subtrees
+                    ($template, $token, $iamLeaf) ? $ok:0;
             }
         }
     }
@@ -376,6 +354,7 @@ sub compile_subtrees
         my $value = $setvar->getAttribute('value');
         if( not defined( $name ) or not defined( $value ) )
         {
+            my $path = $self->path($token);
             Error("Setvar statement without name or value in $path"); $ok = 0;
         }
         else
@@ -391,11 +370,12 @@ sub compile_subtrees
         my $var = $iftrue->getAttribute('var');
         if( not defined( $var ) )
         {
+            my $path = $self->path($token);
             Error("Iftrue statement without variable name in $path"); $ok = 0;
         }
         elsif( $self->isTrueVar( $token, $var ) )
         {
-            $ok = $self->compile_subtrees( $iftrue, $token ) ? $ok:0;
+            $ok = $self->compile_subtrees( $iftrue, $token, $iamLeaf ) ? $ok:0;
         }
     }
 
@@ -404,11 +384,13 @@ sub compile_subtrees
         my $var = $iffalse->getAttribute('var');
         if( not defined( $var ) )
         {
+            my $path = $self->path($token);
             Error("Iffalse statement without variable name in $path"); $ok = 0;
         }
         elsif( not $self->isTrueVar( $token, $var ) )
         {
-            $ok = $self->compile_subtrees( $iffalse, $token ) ? $ok:0;
+            $ok = $self->compile_subtrees
+                ( $iffalse, $token, $iamLeaf ) ? $ok:0;
         }
     }
 
@@ -422,11 +404,11 @@ sub compile_subtrees
             my $name = $subtree->getAttribute('name');
             if( not defined( $name ) or length( $name ) == 0 )
             {
+                my $path = $self->path($token);
                 Error("Subtree without a name at $path"); $ok = 0;
             }
             else
             {
-                $name = $self->expand_name( $path, $name );
                 if( $self->validate_nodename( $name ) )
                 {
                     my $stoken = $self->addChild($token, $name.'/');
@@ -434,6 +416,7 @@ sub compile_subtrees
                 }
                 else
                 {
+                    my $path = $self->path($token);
                     Error("Invalid subtree name: $name at $path"); $ok = 0;
                 }
             }
@@ -444,18 +427,19 @@ sub compile_subtrees
             my $name = $leaf->getAttribute('name');
             if( not defined( $name ) or length( $name ) == 0 )
             {
+                my $path = $self->path($token);
                 Error("Leaf without a name at $path"); $ok = 0;
             }
             else
             {
-                $name = $self->expand_name( $path, $name );
                 if( $self->validate_nodename( $name ) )
                 {
                     my $ltoken = $self->addChild($token, $name);
-                    $ok = $self->compile_subtrees( $leaf, $ltoken ) ? $ok:0;
+                    $ok = $self->compile_subtrees( $leaf, $ltoken, 1 ) ? $ok:0;
                 }
                 else
                 {
+                    my $path = $self->path($token);
                     Error("Invalid leaf name: $name at $path"); $ok = 0;
                 }
             }
