@@ -1,4 +1,4 @@
-#  Copyright (C) 2002  Stanislav Sinyagin
+#  Copyright (C) 2002-2010  Stanislav Sinyagin
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: DevDiscover.pm,v 1.34 2008/06/15 19:21:07 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Core SNMP device discovery module
@@ -76,6 +76,7 @@ our @copyParams =
       'domain-name',
       'monitor-period',
       'monitor-timeoffset',
+      'nodeid-device',
       'show-recursive',
       'snmp-host',
       'snmp-port',
@@ -97,6 +98,7 @@ our @copyParams =
       'snmp-oids-per-pdu',
       'snmp-check-sysuptime',
       'snmp-max-msg-size',
+      'snmp-reachability-rra',
       'system-id' );
 
 
@@ -198,6 +200,15 @@ sub discover
             return 0;
         }
         $snmpargs{'-community'} = $community;
+
+        # set maxMsgSize to a maximum value for better compatibility
+        
+        my $maxmsgsize = $devdetails->param('snmp-max-msg-size');
+        if( defined( $maxmsgsize ) )
+        {
+            $devdetails->setParam('snmp-max-msg-size', $maxmsgsize);
+            $snmpargs{'-maxmsgsize'} = $maxmsgsize;
+        }        
     }
     elsif( $version eq '3' )        
     {
@@ -232,6 +243,8 @@ sub discover
     $snmpargs{'-hostname'} = $hostname;
 
     my $port = $snmpargs{'-port'};
+
+    my $time_start = time();
     Debug('Discovering host: ' . $hostname . ':' . $port . ':' . $community);
 
     my ($session, $error) =
@@ -243,7 +256,7 @@ sub discover
         Error('Cannot create SNMP session: ' . $error);
         return undef;
     }
-
+    
     my @oids = ();
     foreach my $var ( @systemOIDs )
     {
@@ -299,17 +312,6 @@ sub discover
              'value' => $devdetails->snmpVar($self->oiddef('sysDescr'))
              }
          );
-
-    if( defined( $devdetails->snmpVar($self->oiddef('sysUpTime')) ) )
-    {
-        $legendValues{40}{'name'} = 'Uptime';
-        $legendValues{40}{'value'} =
-            sprintf("%d days since %s",
-                    $devdetails->snmpVar($self->oiddef('sysUpTime')) /
-                    (100*3600*24),
-                    strftime($Torrus::DevDiscover::timeFormat,
-                             localtime(time())));
-    }
      
     my $legend = '';
     foreach my $key ( sort keys %legendValues )
@@ -317,7 +319,7 @@ sub discover
         my $text = $legendValues{$key}{'value'};
         if( length( $text ) > 0 )
         {
-            $text = $self->screenSpecialChars( $text );
+            $text = $devdetails->screenSpecialChars( $text );
             $legend .= $legendValues{$key}{'name'} . ':' . $text . ';';
         }
     }
@@ -432,6 +434,8 @@ sub discover
         foreach my $pair ( split(/\s*;\s*/, $define_tokensets ) )
         {
             my( $tset, $description ) = split( /\s*:\s*/, $pair );
+            my $params = {};
+            
             if( $tset !~ /^[a-z][a-z0-9-_]*$/ )
             {
                 Error('Invalid name for tokenset: ' . $tset);
@@ -444,10 +448,25 @@ sub discover
             }
             else
             {
-                $self->{'define-tokensets'}{$tset} = $description;
+                $params->{'comment'} = $description;
+            }
+
+            my $v = $devdetails->param($tset . '-tokenset-rrgraph-view');
+            if( defined($v) and length($v) > 0 )
+            {
+                $params->{'rrgraph-view'} = $v;
+            }
+
+            if( $ok )
+            {
+                $self->{'define-tokensets'}{$tset} = $params;
             }
         }
     }
+
+    Verbose('Discovery for ' . $hostname . ' finished in ' .
+            (time() - $time_start) . ' seconds');
+    
     return $ok;
 }
 
@@ -518,6 +537,17 @@ sub buildConfig
             {
                 push( @{$data->{'templates'}}, '::holt-winters-defaults' );
             }
+            
+            if( $devdetails->param('disable-reachability-stats') ne 'yes'
+                and
+                (
+                 (not defined($devdetails->param('only-devtypes')))
+                 or
+                 $devdetails->param('enable-reachability-stats') eq 'yes'
+                ) )
+            {
+                push( @{$data->{'templates'}}, '::snmp-reachability' );
+            }
 
             
             my $devNodeName = $devdetails->param('symbolic-name');
@@ -583,8 +613,8 @@ sub buildConfig
         my $tsetsNode = $cb->startTokensets();
         foreach my $tset ( sort keys %{$self->{'define-tokensets'}} )
         {
-            $cb->addTokenset( $tsetsNode, $tset, {
-                'comment' => $self->{'define-tokensets'}{$tset} } );
+            $cb->addTokenset( $tsetsNode, $tset, 
+                              $self->{'define-tokensets'}{$tset} );
         }
     }
 }
@@ -701,7 +731,9 @@ sub checkSnmpOID
 
     my $result = $session->get_request( -varbindlist => [ $oid ] );
     if( $session->error_status() == 0 and
-        defined( $result ) and defined( $result->{$oid} ) )
+        defined($result) and
+        defined($result->{$oid}) and
+        length($result->{$oid}) > 0 )
     {
         return 1;
     }
@@ -753,19 +785,27 @@ sub oidBaseMatch
     return Net::SNMP::oid_base_match( $base_oid, $oid );
 }
 
+##
+# some discovery modules need to adjust max-msg-size
 
-sub screenSpecialChars
+sub setMaxMsgSize
 {
     my $self = shift;
-    my $txt = shift;
+    my $devdetails = shift;
+    my $msgsize = shift;
+    my $opt = shift;
 
-    $txt =~ s/:/{COLON}/gm;
-    $txt =~ s/;/{SEMICOL}/gm;
-    $txt =~ s/%/{PERCENT}/gm;
+    $opt = {} unless defined($opt);
 
-    return $txt;
+    if( (not $opt->{'only_v1_and_v2'}) or $self->session()->version() != 3 )
+    {
+        $self->session()->max_msg_size($msgsize);
+        $devdetails->data()->{'param'}{'snmp-max-msg-size'} = $msgsize;
+    }
 }
 
+    
+
 
 ###########################################################################
 ####  Torrus::DevDiscover::DevDetails: the information container for a device
@@ -989,6 +1029,19 @@ sub data
 }
 
 
+sub screenSpecialChars
+{
+    my $self = shift;
+    my $txt = shift;
+
+    $txt =~ s/:/{COLON}/gm;
+    $txt =~ s/;/{SEMICOL}/gm;
+    $txt =~ s/%/{PERCENT}/gm;
+
+    return $txt;
+}
+
+
 sub applySelectors
 {
     my $self = shift;
