@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoFirewall.pm,v 1.4 2006/07/31 08:59:50 ssinyagin Exp $
+# $Id$
 # Shawn Ferry <lalartu at obscure dot org> <sferry at sevenspace dot com>
 
 # Cisco Firewall devices discovery
@@ -86,17 +86,13 @@ sub discover
         $data->{'param'}{'snmp-oids-per-pdu'} = $oidsPerPDU;
     }
 
-    $session->get_request( -varbindlist =>
-                           [ $dd->oiddef('cfwConnectionStatMax') ] );
-    if( $session->error_status() == 0 )
+    if( $dd->checkSnmpOID('cfwConnectionStatMax') )
     {
         $devdetails->setCap('CiscoFirewall::connections');
     }
-
+    
     # I have not seen a system that supports this.
-    $session->get_request( -varbindlist =>
-                           [ $dd->oiddef('cfwBasicEventsTableLastRow') ] );
-    if( $session->error_status() == 0 )
+    if( $dd->checkSnmpOID('cfwBasicEventsTableLastRow') )
     {
         $devdetails->setCap('CiscoFirewall::events');
     }
