@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoGeneric.pm,v 1.10 2008/06/02 23:28:02 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Common Cisco MIBs, supported by many IOS and CatOS devices
@@ -278,6 +278,7 @@ sub discover
                 if( $phyReferers{$phyIndex} > 1 )
                 {
                     $phyDescr .= ' (' . $INDEX . ')';
+                    $cpuNick .= '_' . $INDEX;
                 }
                 
                 $data->{'ciscoCpuStats'}{$INDEX} = {
@@ -299,9 +300,7 @@ sub discover
             # Although OLD-CISCO-CPU-MIB is implemented in IOS only,
             # it is easier to leave it here in Generic
 
-            $session->get_request( -varbindlist =>
-                                   [ $dd->oiddef('avgBusy1') ] );
-            if( $session->error_status() == 0 )
+            if( $dd->checkSnmpOID('avgBusy1') )
             {
                 $devdetails->setCap('old-ciscoCpuStats');
                 push( @{$data->{'templates'}}, 'CiscoGeneric::old-cisco-cpu' );
@@ -331,7 +330,9 @@ sub buildConfig
         my $fahrenheit =
             $devdetails->param('CiscoGeneric::use-fahrenheit') eq 'yes';
 
-        my $param = {};
+        my $param = {
+            'node-display-name' => 'Temperature Sensors',
+        };
         my $templates = [ 'CiscoGeneric::cisco-temperature-subtree' ];
         
         my $filePerSensor =
@@ -393,8 +394,11 @@ sub buildConfig
         # Create a subtree for the power supplies
         my $subtreeName = 'Power_Supplies';
 
-        my $param = {'comment' => 'Power supplies status',
-                     'precedence' => -600};
+        my $param = {
+            'node-display-name' => 'Power Supplies',
+            'comment' => 'Power supplies status',
+            'precedence' => -600,
+        };
         my $templates = [];
                 
         $param->{'data-file'} = '%system-id%_power.rrd';
@@ -431,8 +435,9 @@ sub buildConfig
         my $subtreeName = 'Memory_Usage';
 
         my $param = {
+            'node-display-name' => 'Memory Usage',
             'precedence'        => '-100',
-            'comment'           => 'Memory usage statistics'
+            'comment'           => 'Router memory utilization'
             };
 
         my $subtreeNode =
@@ -520,6 +525,7 @@ sub buildConfig
     {
         my $subtreeName = 'CPU_Usage';
         my $param = {
+            'node-display-name' => 'CPU Usage',
             'precedence'         => '-500',
             'comment'            => 'Overall CPU busy percentage'
             };
@@ -542,10 +548,12 @@ sub buildConfig
             if( $cpu->{'phy-referers'} > 1 )
             {
                 $param->{'cisco-cpu-indexmap'} = $INDEX;
+                $param->{'cisco-cpu-ref'} = $INDEX;
             }
             else
             {
                 $param->{'entity-phy-index'} = $cpu->{'phy-index'};
+                $param->{'cisco-cpu-ref'} = '%entity-phy-index%';
             }
             
             my @templates;
