@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoIOS.pm,v 1.15 2008/11/29 17:01:27 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Cisco IOS devices discovery
@@ -33,7 +33,7 @@ $Torrus::DevDiscover::registry{'CiscoIOS'} = {
     'checkdevtype' => \&checkdevtype,
     'discover'     => \&discover,
     'buildConfig'  => \&buildConfig
-    };
+};
 
 
 our %oiddef =
@@ -64,8 +64,12 @@ our %oiddef =
      'ccarConfigConformAction'           => '1.3.6.1.4.1.9.9.113.1.1.1.1.8',
      'ccarConfigExceedAction'            => '1.3.6.1.4.1.9.9.113.1.1.1.1.9',
      # CISCO-VPDN-MGMT-MIB
-     'cvpdnSystemTunnelTotal'            => '1.3.6.1.4.1.9.10.24.1.1.4.1.2'
-     );
+     'cvpdnSystemTunnelTotal'            => '1.3.6.1.4.1.9.10.24.1.1.4.1.2',
+     # CISCO-WAN-3G-MIB
+     'c3gStandard'                       => '1.3.6.1.4.1.9.9.661.1.1.1.1',
+     # CISCO-PORT-QOS-MIB
+     'cportQosDropPkts'                  => '1.3.6.1.4.1.9.9.189.1.3.2.1.7',
+    );
 
 
 # Not all interfaces are normally needed to monitor.
@@ -91,66 +95,61 @@ if( not defined( $interfaceFilter ) )
      'Null0' => {
          'ifType'  => 1,                      # other
          'ifDescr' => '^Null'
-         },
+     },
 
      'E1 N/N/N' => {
          'ifType'  => 18,                     # ds1
          'ifDescr' => '^E1'
-         },
+     },
 
      'Virtual-AccessN' => {
          'ifType'  => 23,                     # ppp
          'ifDescr' => '^Virtual-Access'
-         },
+     },
      
-     'DialerN' => {
-         'ifType'  => 23,                     # ppp
-         'ifDescr' => '^Dialer'
-         },
-
      'LoopbackN'  => {
          'ifType'  => 24,                     # softwareLoopback
          'ifDescr' => '^Loopback'
-         },
-
-     'unrouted VLAN N' => {
-         'ifType'  => 53,                     # propVirtual
-         'ifDescr' => '^unrouted\s+VLAN\s+\d+'
-         },
+     },
 
      'SerialN:N-Bearer Channel' => {
          'ifType'  => 81,                     #  ds0, Digital Signal Level 0
          'ifDescr' => '^Serial.*Bearer\s+Channel'
-         },
+     },
 
      'Voice Encapsulation (POTS) Peer: N' => {
          'ifType'  => 103                     # voiceEncap
-         },
+     },
 
      'Voice Over IP Peer: N' => {
          'ifType'  => 104                     # voiceOverIp
-         },
+     },
 
      'ATMN/N/N.N-atm subif' => {
          'ifType'  => 134,                    # atmSubInterface
          'ifDescr' => '^ATM[0-9\/]+\.[0-9]+\s+subif'
-         },
+     },
      
      'BundleN' => {
          'ifType'  => 127,                    # docsCableMaclayer
          'ifDescr' => '^Bundle'
-         },
+     },
 
      'EOBCN/N' => {
          'ifType'  => 53,                     # propVirtual
          'ifDescr' => '^EOBC'
-         },
+     },
 
      'FIFON/N' => {
          'ifType'  => 53,                     # propVirtual
          'ifDescr' => '^FIFO'
-         },
-     );
+     },
+     
+     'vfc' => {
+         'ifType'  => 56,                     # virtualFibreChannel
+         'ifDescr' => '^vfc'
+     },
+    );
 
 our %tunnelType =
     (
@@ -195,9 +194,29 @@ sub checkdevtype
         $interfaceFilter->{'VlanN'} = {
             'ifType'  => 53,                     # propVirtual
             'ifDescr' => '^Vlan\d+'
-            };
+        };
     }
-        
+
+    # same thing with unrouted VLAN interfaces
+    if( $devdetails->param('CiscoIOS::enable-unrouted-vlan-interfaces')
+        ne 'yes' )
+    {
+        $interfaceFilter->{'unrouted VLAN N'} = {
+            'ifType'  => 53,                     # propVirtual
+            'ifDescr' => '^unrouted\s+VLAN\s+\d+'
+        };
+    }
+
+    # sometimes we want Dialer interface stats
+    if( $devdetails->param('CiscoIOS::enable-dialer-interfaces')
+        ne 'yes' )
+    {
+        $interfaceFilter->{'DialerN'} = {
+            'ifType'  => 23,                     # ppp
+            'ifDescr' => '^Dialer'
+        },
+    }
+
     &Torrus::DevDiscover::RFC2863_IF_MIB::addInterfaceFilter
         ($devdetails, $interfaceFilter);
 
@@ -224,7 +243,7 @@ my %ccarAction =
       3 => 'continue',
       4 => 'precedXmit',
       5 => 'precedCont' );
-                      
+
 
 
 sub discover
@@ -261,14 +280,14 @@ sub discover
                                          . '.' . $ifIndex ) == 0 and
                    $devdetails->snmpVar( $dd->oiddef('ifInOctets')
                                          . '.' . $ifIndex ) > 0
-                   )
+                  )
                   or
                   (
                    $devdetails->snmpVar( $dd->oiddef('ifHCOutOctets')
                                          . '.' . $ifIndex ) == 0 and
                    $devdetails->snmpVar( $dd->oiddef('ifOutOctets')
                                          . '.' . $ifIndex ) > 0
-                   ) ) )
+                  ) ) )
             {
                 Debug('Disabling HC octets for ' . $ifIndex . ': ' .
                       $interface->{'ifDescr'});
@@ -284,9 +303,7 @@ sub discover
         # Old Memory Buffers, if we have bufferElFree we assume
         # the rest as they are "required"
 
-        $session->get_request( -varbindlist =>
-                               [ $dd->oiddef('bufferElFree') ] );
-        if( $session->error_status() == 0 )
+        if( $dd->checkSnmpOID('bufferElFree') )
         {
             $devdetails->setCap('old-ciscoMemoryBuffers');
             push( @{$data->{'templates'}},
@@ -296,9 +313,7 @@ sub discover
 
     if( $devdetails->param('CiscoIOS::disable-ipsec-stats') ne 'yes' )
     {
-        $session->get_request( -varbindlist =>
-                               [ $dd->oiddef('cipSecGlobalHcInOctets') ] );
-        if( $session->error_status() == 0 )
+        if( $dd->checkSnmpOID('cipSecGlobalHcInOctets') )
         {
             $devdetails->setCap('ciscoIPSecGlobalStats');
             push( @{$data->{'templates'}},
@@ -349,7 +364,7 @@ sub discover
                 my $peer = {
                     'peerIP' => $peerIP,
                     'addrFamily' => 'IPv4 Unicast'
-                    };
+                };
                 
                 if( $afi != 1 and $safi != 1 )
                 {
@@ -397,7 +412,7 @@ sub discover
                 $peer->{'prefixLimit'} =
                     $limitsTable->{$dd->oiddef('cbgpPeerPrefixAdminLimit') .
                                        '.' . $INDEX};
-                    
+                
                 $data->{'cbgpPeers'}{$INDEX} = $peer;                
             }
 
@@ -513,21 +528,21 @@ sub discover
         {
             # Find the Tunnel type
             my $tableTun = $session->get_table(
-                            -baseoid => $dd->oiddef('cvpdnSystemTunnelTotal') );
+                -baseoid => $dd->oiddef('cvpdnSystemTunnelTotal') );
 
             if( $tableTun )
             {
-                $devdetails->setCap("ciscoVPDN");
+                $devdetails->setCap('ciscoVPDN');
 
                 $devdetails->storeSnmpVars( $tableTun );
 
                 # VPDN indexing: 1: l2f, 2: l2tp, 3: pptp
                 foreach my $typeIndex (
-                            $devdetails->getSnmpIndices(
-                              $dd->oiddef('cvpdnSystemTunnelTotal') ) )
+                    $devdetails->getSnmpIndices(
+                        $dd->oiddef('cvpdnSystemTunnelTotal') ) )
                 {
                     Debug("CISCO-VPDN-MGMT-MIB: found Tunnel type " .
-                           $tunnelType{$typeIndex} );
+                          $tunnelType{$typeIndex} );
 
                     $data->{'ciscoVPDN'}{$typeIndex} = $tunnelType{$typeIndex};
                 }
@@ -535,6 +550,74 @@ sub discover
         }
     }
 
+    if( $devdetails->param('CiscoIOS::disable-3g-stats') ne 'yes' )
+    {
+        my $base = $dd->oiddef('c3gStandard');
+        my $table = $session->get_table( -baseoid => $base );
+        my $prefixLen = length( $base ) + 1;
+        
+        if( defined( $table ) and scalar( %{$table} ) > 0 )
+        {            
+            while( my( $oid, $val ) = each %{$table} )
+            {
+                if( $val == 2 )
+                {
+                    $devdetails->setCap('Cisco3G');
+                    my $phy = substr( $oid, $prefixLen );
+                    my $phyName = $data->{'entityPhysical'}{$phy}{'name'};
+                    if( not defined( $phyName ) )
+                    {
+                        $phyName = 'Cellular Modem';
+                    }
+                    
+                    $data->{'cisco3G'}{$phy} = {
+                        'name'   => $phyName,
+                        'standard' => 'gsm',
+                    };
+                }
+                else
+                {
+                    Warn('CISCO-WAN-3G-MIB: CDMA statistics are ' .
+                         'not yet implemented');
+                }
+            }
+        }
+
+        my $tokenset = $devdetails->param('CiscoIOS::3g-stats-tokenset');
+        if( defined($tokenset) )
+        {
+            $data->{'cisco3G_tokenset'} = $tokenset;
+        }
+    }
+
+
+    if( $devdetails->param('CiscoIOS::disable-port-qos-stats') ne 'yes' )
+    {
+        my $base = $dd->oiddef('cportQosDropPkts');
+        my $table = $session->get_table( -baseoid => $base );
+        my $prefixLen = length( $base ) + 1;
+        
+        if( defined( $table ) and scalar( %{$table} ) > 0 )
+        {            
+            $devdetails->setCap('CiscoPortQos');
+            $data->{'ciscoPortQos'} = {};
+            
+            while( my( $oid, $val ) = each %{$table} )
+            {
+                my $qos_entry = substr( $oid, $prefixLen );
+                my($ifIndex, $direction, $qos_index) =
+                    split(/\./o, $qos_entry);
+                
+                $data->{'ciscoPortQos'}{$qos_entry} = {
+                    'ifIndex' => $ifIndex,
+                    'direction' => ($direction==1)?'ingress':'egress',
+                    'qosidx' => $qos_index,
+                };                
+            }
+        }
+    }
+    
+    
     if( $devdetails->param('CiscoIOS::short-device-comment') eq 'yes' )
     {
         # Remove serials from device comment
@@ -559,7 +642,10 @@ sub buildConfig
     {
         my $countersNode =
             $cb->addSubtree( $devNode, 'BGP_Prefixes',
-                             { 'comment' => 'Accepted prefixes'} );
+                             {
+                                 'node-display-name' => 'BGP Prefixes',
+                                 'comment' => 'Accepted prefixes',
+                             } );
 
         foreach my $INDEX ( sort
                             { $data->{'cbgpPeers'}{$a}{'subtreeName'} <=>
@@ -593,9 +679,9 @@ sub buildConfig
     if( $devdetails->hasCap('CiscoCAR') )
     {
         my $countersNode =
-            $cb->addSubtree( $devNode, 'CAR_Stats',
-                             { 'comment' =>
-                                   'Committed Access Rate statistics'},
+            $cb->addSubtree( $devNode, 'CAR_Stats', {
+                'comment' => 'Committed Access Rate statistics',
+                'node-display-name' => 'CAR', },
                              ['CiscoIOS::cisco-car-subtree']);
         
         foreach my $INDEX ( sort keys %{$data->{'ccar'}} )
@@ -611,12 +697,12 @@ sub buildConfig
             {
                 $subtreeName .= '_' . $car->{'carIndex'};
             }
-               
+            
             my $param = {
                 'searchable' => 'yes',
                 'car-direction' => $car->{'direction'},
                 'car-index' => $car->{'carIndex'} };
-                
+            
             $param->{'interface-name'} =
                 $interface->{'param'}{'interface-name'};            
             $param->{'interface-nick'} =
@@ -652,19 +738,103 @@ sub buildConfig
 
     if( $devdetails->hasCap('ciscoVPDN') )
     { 
-        my $tunnelNode = $cb->addSubtree( $devNode, 'VPDN_Statistics',
-                             { 'comment' => 'VPDN statistics' },
-                             [ 'CiscoIOS::cisco-vpdn-subtree' ] );
+        my $tunnelNode = $cb->addSubtree
+            ( $devNode, 'VPDN_Statistics',
+              {'node-display-name' => 'VPDN Statistics'},
+              [ 'CiscoIOS::cisco-vpdn-subtree' ] );
 
         foreach my $INDEX ( sort keys %{$data->{'ciscoVPDN'}} )
         {
             my $tunnelProtocol = $data->{'ciscoVPDN'}{$INDEX};
 
             $cb->addSubtree( $tunnelNode, $tunnelProtocol,
-                { 'comment'  => $tunnelProtocol . ' information',
-                  'tunIndex' => $INDEX,
-                  'tunFile'  => lc($tunnelProtocol) },
-                [ 'CiscoIOS::cisco-vpdn-leaf' ] );
+                             { 'comment'  => $tunnelProtocol . ' information',
+                               'tunIndex' => $INDEX,
+                               'tunFile'  => lc($tunnelProtocol) },
+                             [ 'CiscoIOS::cisco-vpdn-leaf' ] );
+        }
+    }
+
+    if( $devdetails->hasCap('Cisco3G') )
+    { 
+        foreach my $phy ( sort keys %{$data->{'cisco3G'}} )
+        {
+            my @templates;
+            my $summary_leaf;
+            if( $data->{'cisco3G'}{$phy}{'standard'} eq 'gsm' )
+            {
+                push(@templates, 'CiscoIOS::cisco-3g-gsm-stats');
+                $summary_leaf = 'RSSI';
+            }
+            
+            my $phyName = $data->{'cisco3G'}{$phy}{'name'};
+
+            my $subtreeName = $phyName;
+            $subtreeName =~ s/\W/_/go;
+            
+            my $subtree = $cb->addSubtree
+                ( $devNode, $subtreeName,
+                  {
+                      'node-display-name' => $phyName,
+                      'comment'  => '3G cellular statistics',
+                      'cisco-3g-phy' => $phy,
+                      'cisco-3g-phyname' => $phyName,
+                      '3gcell-nodeid' => 'cell//%nodeid-device%//' . $phy,
+                      'nodeid' => '%3gcell-nodeid%',
+                  },
+                  \@templates );
+            
+            if( defined($data->{'cisco3G_tokenset'}) )
+            {
+                $cb->addLeaf
+                    ($subtree, $summary_leaf,
+                     {'tokenset-member' => $data->{'cisco3G_tokenset'}});
+            }
+        }
+    }
+
+    if( $devdetails->hasCap('CiscoPortQos') )
+    {
+        my $subtreeNode = $cb->addSubtree
+            ( $devNode, 'Port_QoS',
+              {'node-display-name' => 'QoS drop statistics'} );
+
+        my $precedence = 1000;
+        foreach my $qos_entry (sort keys %{$data->{'ciscoPortQos'}})
+        {
+            my $entry = $data->{'ciscoPortQos'}{$qos_entry};
+            
+            my $interface = $data->{'interfaces'}{$entry->{'ifIndex'}};
+            next unless defined($interface);
+            next if $interface->{'excluded'};
+
+            my $leafName =
+                $interface->{$data->{'nameref'}{'ifSubtreeName'}} . '_' .
+                $entry->{'direction'} . '_drops';
+
+            my $intfName  =
+                $interface->{$data->{'nameref'}{'ifReferenceName'}};
+
+            my $intfNick = $interface->{$data->{'nameref'}{'ifNick'}};
+
+            my $params = {
+                'cisco-pqos-entry' => $qos_entry,
+                'cisco-pqos-ifnick' => $intfNick,
+                'cisco-pqos-direction' => $entry->{'direction'},
+                'graph-legend' => 'Dropped packets',
+                'precedence' => $precedence--,                
+            };
+
+            $params->{'node-display-name'} =
+                $intfName . ' ' . $entry->{'direction'} . ' drops';
+            
+            $params->{'graph-title'} =
+                $params->{'descriptive-nickname'} = 
+                '%system-id% ' . $intfName . ' ' .
+                $entry->{'direction'} . ' drops';
+            
+            $cb->addLeaf($subtreeNode, $leafName, $params,
+                ['CiscoIOS::cisco-port-qos-packets']);
         }
     }
 }
