@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: CiscoIOS_MacAccounting.pm,v 1.10 2008/06/19 22:09:24 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Cisco IOS MAC accounting
@@ -289,7 +289,7 @@ sub buildConfig
 
     my $countersNode =
         $cb->addSubtree( $devNode, 'MAC_Accounting',
-                         { 'comment' => 'MAC accounting statistics'},
+                         {'node-display-name' => 'MAC Accounting'},
                          ['CiscoIOS_MacAccounting::cisco-macacc-subtree']);
     
     foreach my $INDEX ( sort { $data->{'cipMac'}{$a}{'subtreeName'} <=>
