@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: F5BigIp.pm,v 1.2 2005/01/25 13:46:34 ssinyagin Exp $
+# $Id$
 # Shawn Ferry <sferry at sevenspace dot com> <lalartu at obscure dot org>
 
 # F5 BigIp Load Balancer
@@ -149,24 +149,17 @@ sub discover
 
     # this is rather basic, per-capability checking
     # may be required in the future
-    $session->get_request( -varbindlist =>
-                           [ $dd->oiddef('4.x_globalStatUptime') ] );
-    if( $session->error_status() == 0 )
+
+    if( $dd->checkSnmpOID('4.x_globalStatUptime') )
     {
         $devdetails->setCap('BigIp_4.x');
     }
-    else
+    elsif( $dd->checkSnmpOID('3.x_uptime') )
     {
-        $session->get_request( -varbindlist =>
-                        [ $dd->oiddef('3.x_uptime') ] );
-
-        if( $session->error_status() == 0 )
-        {
-            # for v3.x we are not supporting detailed stats, so don't check
-            # anything else
-            $devdetails->setCap('BigIp_3.x');
-            return 1;
-        }
+        # for v3.x we are not supporting detailed stats, so don't check
+        # anything else
+        $devdetails->setCap('BigIp_3.x');
+        return 1;
     }
 
     my $product_name;
@@ -174,8 +167,8 @@ sub discover
     my $result = $dd->retrieveSnmpOIDs( '4.x_globalAttrProductCode' );
     my $product_code = $result->{'4.x_globalAttrProductCode'};
 
-    $product_name = %f5_product->{$product_code}->{'product'};
-    if( %f5_product->{$product_code}->{'supported'} )
+    $product_name = $f5_product{$product_code}->{'product'};
+    if( $f5_product{$product_code}->{'supported'} )
     {
         $devdetails->setCap( 'BigIp_' . $product_name );
     }
