@@ -15,7 +15,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: JunOS.pm,v 1.5 2008/01/12 22:46:06 ssinyagin Exp $
+# $Id$
 # Jon Nistor <nistor at snickers.org>
 
 # Juniper JunOS Discovery Module
@@ -223,13 +223,15 @@ sub discover
                 $session->get_table( -baseoid =>
                                      $dd->oiddef('jnxCosQstatQedPkts') );
             $devdetails->storeSnmpVars( $cosIfIndex );
-            
-            foreach my $INDEX ( $devdetails->getSnmpIndices
-                                ($dd->oiddef('jnxCosQstatQedPkts')) )
+
+	    if( $cosIfIndex )            
             {
-                my( $ifIndex, $cosQueueIndex ) = split( '\.', $INDEX );
-                $data->{'jnxCos'}{'ifIndex'}{$ifIndex} = 1; # 1 = placeholder
-                
+		foreach my $INDEX ( $devdetails->getSnmpIndices
+                                ($dd->oiddef('jnxCosQstatQedPkts')) )
+            	{
+                	my( $ifIndex, $cosQueueIndex ) = split( '\.', $INDEX );
+			$data->{'jnxCos'}{'ifIndex'}{$ifIndex} = 1;
+               	} 
             }
         }
     } # END JunOS::disable-cos
@@ -296,7 +298,6 @@ sub discover
                 $data->{'jnxOperating'}{$opIndex}{'isr'}   = $opISR;
                 $data->{'jnxOperating'}{$opIndex}{'mem'}   = $opMem;
                 $data->{'jnxOperating'}{$opIndex}{'temp'}  = $opTemp;
-
             }
         } # END: if $tableDesc
     } # END: JunOS::disable-operating
@@ -373,21 +374,24 @@ sub discover
                                    ($dd->oiddef('jnxRpfStatsPackets')) )
             {
                 my ($ifIndex,$addrFamily) = split('\.',$rpfIndex);
-                my $ifAddrFam = $addrFamily == 1 ? 'ipv4' : 'ipv6';
-                my $intName   = $data->{'interfaces'}{$ifIndex}{'ifName'};
-                my $intNameT  = $data->{'interfaces'}{$ifIndex}{'ifNameT'};
-
-                # Construct data
-                $data->{'jnxRPF'}{$ifIndex}{'ifName'}  = $intName;
-                $data->{'jnxRPF'}{$ifIndex}{'ifNameT'} = $intNameT;
-
-                if( $addrFamily == 1 )
+                if( defined( $data->{'interfaces'}{$ifIndex} ) )
                 {
-                    $data->{'jnxRPF'}{$ifIndex}{'ipv4'} = 1;
-                }
-                if( $addrFamily == 2 )
-                {
-                    $data->{'jnxRPF'}{$ifIndex}{'ipv6'} = 2;
+                    my $ifAddrFam = $addrFamily == 1 ? 'ipv4' : 'ipv6';
+                    my $intName   = $data->{'interfaces'}{$ifIndex}{'ifName'};
+                    my $intNameT  = $data->{'interfaces'}{$ifIndex}{'ifNameT'};
+                    
+                    # Construct data
+                    $data->{'jnxRPF'}{$ifIndex}{'ifName'}  = $intName;
+                    $data->{'jnxRPF'}{$ifIndex}{'ifNameT'} = $intNameT;
+                    
+                    if( $addrFamily == 1 )
+                    {
+                        $data->{'jnxRPF'}{$ifIndex}{'ipv4'} = 1;
+                    }
+                    if( $addrFamily == 2 )
+                    {
+                        $data->{'jnxRPF'}{$ifIndex}{'ipv6'} = 2;
+                    }                    
                 }
             }
         }
@@ -406,10 +410,13 @@ sub buildConfig
 
 
     # PROG: Class of Service information
-    if( $devdetails->hasCap('jnxCoS') )
+    if( $devdetails->hasCap('jnxCoS') &&
+	( keys %{$data->{'jnxCos'}{'ifIndex'}} > 0 )
+       )
     {
-        my $nodeTop = $cb->addSubtree( $devNode, 'CoS', undef,
-                                       [ 'JunOS::junos-cos-subtree']);
+	# PROG: Add CoS information if it exists.
+	my $nodeTop = $cb->addSubtree( $devNode, 'CoS', undef,
+                                  [ 'JunOS::junos-cos-subtree']);
 
         foreach my $ifIndex ( sort {$a <=> $b} keys
                               %{$data->{'jnxCos'}{'ifIndex'}} )
@@ -419,7 +426,7 @@ sub buildConfig
             my $ifDescr   = $interface->{'ifDescr'};
             my $ifName    = $interface->{'ifNameT'};
 
-            next if (!$ifName);  # Skip to next since port is likely 'disabled'
+            next if( not $ifName );  # Skip since port is likely 'disabled'
             # This might be better to match against ifType
             # as well since not all of them support Q's.
 
@@ -552,8 +559,8 @@ sub buildConfig
             my $jnxMem    = $operating->{'mem'};
             my $jnxTemp   = $operating->{'temp'};
             my $jnxTag = $jnxDesc;
-            $jnxTag =~ s/[:\/\*\@\(\), ]+/_/g;
-            $jnxTag =~ s/[_]$//g;
+            $jnxTag =~ s/\W+/_/go;
+            $jnxTag =~ s/_$//go;
             # Fix the .'s into _'s for the RRD-DS and name of leaf
             my $opIndexFix = $opIndex;
             $opIndexFix =~ s/\./_/g;
