@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RFC2737_ENTITY_MIB.pm,v 1.3 2006/02/22 12:47:16 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Discovery module for ENTITY-MIB (RFC 2737)
@@ -97,12 +97,9 @@ sub discover
         {
             my $oid = $oidContainedIn . '.' . $phyIndex;
             my $result = $session->get_request( -varbindlist => [ $oid ] );
-            if( $session->error_status() == 0 )
+            if( $session->error_status() == 0 and $result->{$oid} == 0 )
             {
-                if( $result->{$oid} == 0 )
-                {
-                    $chassisIndex = $phyIndex;
-                }
+                $chassisIndex = $phyIndex;
             }
         }
 
