@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RFC2863_IF_MIB.pm,v 1.40 2008/07/23 21:14:20 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # Standard IF_MIB discovery, which should apply to most devices
@@ -26,7 +26,7 @@ use Torrus::Log;
 
 
 $Torrus::DevDiscover::registry{'RFC2863_IF_MIB'} = {
-    'sequence'     => 10,
+    'sequence'     => 50,
     'checkdevtype' => \&checkdevtype,
     'discover'     => \&discover,
     'buildConfig'  => \&buildConfig,
@@ -39,6 +39,7 @@ our %oiddef =
      'ifTable'          => '1.3.6.1.2.1.2.2',
      'ifDescr'          => '1.3.6.1.2.1.2.2.1.2',
      'ifType'           => '1.3.6.1.2.1.2.2.1.3',
+     'ifSpeed'          => '1.3.6.1.2.1.2.2.1.5',
      'ifPhysAddress'    => '1.3.6.1.2.1.2.2.1.6',
      'ifAdminStatus'    => '1.3.6.1.2.1.2.2.1.7',
      'ifOperStatus'     => '1.3.6.1.2.1.2.2.1.8',
@@ -56,6 +57,7 @@ our %oiddef =
      'ifHCInUcastPkts'  => '1.3.6.1.2.1.31.1.1.1.7',
      'ifHCOutOctets'    => '1.3.6.1.2.1.31.1.1.1.10',
      'ifHCOutUcastPkts' => '1.3.6.1.2.1.31.1.1.1.11',
+     'ifHighSpeed'      => '1.3.6.1.2.1.31.1.1.1.15',     
      'ifAlias'          => '1.3.6.1.2.1.31.1.1.1.18'
      );
 
@@ -67,17 +69,7 @@ sub checkdevtype
     my $dd = shift;
     my $devdetails = shift;
 
-    my $session = $dd->session();
-
-    my $ifTable =
-        $session->get_table( -baseoid => $dd->oiddef('ifTable') );
-    if( not defined $ifTable )
-    {
-        return 0;
-    }
-    $devdetails->storeSnmpVars( $ifTable );
-
-    return 1;
+    return $dd->checkSnmpTable('ifTable');
 }
 
 
@@ -87,22 +79,39 @@ sub discover
     my $devdetails = shift;
     
     my $session = $dd->session();
-
-    my $ifXTable =
-        $session->get_table( -baseoid => $dd->oiddef('ifXTable') );
-    if( defined $ifXTable )
+    
+    my $ifTable =
+        $session->get_table( -baseoid => $dd->oiddef('ifTable') );
+    if( not defined $ifTable )
     {
-        $devdetails->storeSnmpVars( $ifXTable );
-        $devdetails->setCap('ifXTable');
+        Error('Cannot retrieve ifTable');
+        return 0;
+    }
+    $devdetails->storeSnmpVars( $ifTable );
 
-        if( $devdetails->hasOID( $dd->oiddef('ifName') ) )
+    if( not $devdetails->hasCap('disable_ifXTable') )
+    {
+        my $ifXTable =
+            $session->get_table( -baseoid => $dd->oiddef('ifXTable') );
+        if( defined $ifXTable )
         {
-            $devdetails->setCap('ifName');
-        }
-
-        if( $devdetails->hasOID( $dd->oiddef('ifAlias') ) )
-        {
-            $devdetails->setCap('ifAlias');
+            $devdetails->storeSnmpVars( $ifXTable );
+            $devdetails->setCap('ifXTable');
+            
+            if( $devdetails->hasOID( $dd->oiddef('ifName') ) )
+            {
+                $devdetails->setCap('ifName');
+            }
+            
+            if( $devdetails->hasOID( $dd->oiddef('ifAlias') ) )
+            {
+                $devdetails->setCap('ifAlias');
+            }
+            
+            if( $devdetails->hasOID( $dd->oiddef('ifHighSpeed') ) )
+            {
+                $devdetails->setCap('ifHighSpeed');
+            }
         }
     }
 
@@ -117,19 +126,44 @@ sub discover
     $data->{'param'}{'has-inout-leaves'} = 'yes';
 
     ## Set default interface index mapping
-    
-    $data->{'nameref'}{'ifSubtreeName'} = 'ifDescrT';
-    $data->{'nameref'}{'ifReferenceName'}   = 'ifDescr';
 
-    if( $devdetails->hasCap('ifName') )
+    my $valid_ifDescr = validateReference($devdetails, 'ifDescr');
+    my $valid_ifName = ( $devdetails->hasCap('ifName') and
+                         validateReference($devdetails, 'ifName') );
+    
+    if( $valid_ifDescr )
     {
+        $data->{'nameref'}{'ifSubtreeName'} = 'ifDescrT';
+        $data->{'nameref'}{'ifReferenceName'}   = 'ifDescr';
+
+        if( $devdetails->param('RFC2863_IF_MIB::ifnick-from-ifname') eq 'yes'
+            and $devdetails->hasCap('ifName') )
+        {
+            $data->{'nameref'}{'ifNick'} = 'ifNameT';
+        }
+        elsif( $valid_ifName )
+        {
+            $data->{'nameref'}{'ifNick'} = 'ifNameT';
+        }
+        else
+        {
+            $data->{'nameref'}{'ifNick'} = 'ifDescrT';
+        }
+    }
+    elsif( $valid_ifName )
+    {
+        $data->{'nameref'}{'ifSubtreeName'} = 'ifNameT';
+        $data->{'nameref'}{'ifReferenceName'} = 'ifName';
         $data->{'nameref'}{'ifNick'} = 'ifNameT';
     }
     else
     {
-        $data->{'nameref'}{'ifNick'} = 'ifDescrT';
+        $devdetails->setCap('interfaceIndexingPersistent');
+        $data->{'nameref'}{'ifSubtreeName'} = 'ifIndex';
+        $data->{'nameref'}{'ifReferenceName'} = 'ifIndex';
+        $data->{'nameref'}{'ifNick'} = 'ifIndex';
     }
-
+    
     if( $devdetails->hasCap('ifAlias') )
     {
         $data->{'nameref'}{'ifComment'} = 'ifAlias';
@@ -164,6 +198,10 @@ sub discover
             $interface->{'param'} = {};
             $interface->{'vendor_templates'} = [];
 
+            $interface->{'ifIndex'} = $ifIndex;
+            $interface->{'ifAdminStatus'} = $admStatus;
+            $interface->{'ifOperStatus'} = $operStatus;
+            
             $interface->{'ifType'} =
                 $devdetails->snmpVar($dd->oiddef('ifType') . '.' . $ifIndex);
 
@@ -195,6 +233,30 @@ sub discover
                     $devdetails->snmpVar($dd->oiddef('ifAlias') .
                                          '.' . $ifIndex);
             }
+
+            my $bw = 0;
+            if( $devdetails->hasCap('ifHighSpeed') )
+            {
+                my $hiBW = 
+                    $devdetails->snmpVar($dd->oiddef('ifHighSpeed') . '.' .
+                                         $ifIndex);
+                if( $hiBW >= 10 )
+                {
+                    $bw = 1e6 * $hiBW;
+                }
+            }
+
+            if( $bw == 0 )
+            {
+                $bw = 
+                    $devdetails->snmpVar($dd->oiddef('ifSpeed') . '.' .
+                                         $ifIndex);
+            }
+            
+            if( $bw > 0 )
+            {
+                $interface->{'ifSpeed'} = $bw;
+            }
         }
     }
 
@@ -238,12 +300,6 @@ sub discover
                       $hint);
             }
         }
-
-        if( $devdetails->hasCap('interfaceIndexingPersistent') )
-        {
-            $data->{'param'}{'ifindex-map'} = '$IFIDX_IFINDEX';
-            storeIfIndexParams( $devdetails );
-        }
             
         $hint =
             $devdetails->param('RFC2863_IF_MIB::subtree-name-hint');
@@ -259,8 +315,31 @@ sub discover
                       $hint);
             }
         }
+        
+        $hint =
+            $devdetails->param('RFC2863_IF_MIB::nodeid-hint');
+        if( defined( $hint ) )
+        {
+            $data->{'nameref'}{'ifNodeid'} = $hint;
+        }
+    }
+    
+    if( $devdetails->hasCap('interfaceIndexingPersistent') )
+    {
+        $data->{'param'}{'ifindex-map'} = '$IFIDX_IFINDEX';
+        storeIfIndexParams( $devdetails );
     }
 
+    if( not defined( $data->{'nameref'}{'ifNodeid'} ) )
+    {
+        $data->{'nameref'}{'ifNodeid'} = 'ifNodeid';
+    }
+    
+    if( not defined( $data->{'nameref'}{'ifNodeidPrefix'} ) )
+    {
+        $data->{'nameref'}{'ifNodeidPrefix'} = 'ifNodeidPrefix';
+    }
+    
     # Filter out the interfaces if needed
 
     if( ref( $data->{'interfaceFilter'} ) )
@@ -395,6 +474,26 @@ sub buildConfig
 
     uniqueEntries( $devdetails, $data->{'nameref'}{'ifNick'} );
     uniqueEntries( $devdetails, $data->{'nameref'}{'ifSubtreeName'} );
+
+    # If other discovery modules don't set nodeid reference, fall back to
+    # default interface reference
+    
+    # Bandwidth usage
+    my %bandwidthLimits;
+    my $bandwidthUsageConfigured = 0;
+    my $bandwidthUsageActive = 0;
+    
+    if( $devdetails->param('RFC2863_IF_MIB::bandwidth-usage') eq 'yes' )
+    {
+        $bandwidthUsageConfigured = 1;
+        my $limits = $devdetails->param('RFC2863_IF_MIB::bandwidth-limits');
+        foreach my $intfLimit ( split( /\s*;\s*/, $limits ) )
+        {
+            my( $intf, $limitIn, $limitOut ) = split( /\s*:\s*/, $intfLimit );
+            $bandwidthLimits{$intf}{'In'} = $limitIn;
+            $bandwidthLimits{$intf}{'Out'} = $limitOut;
+        }
+    }
     
     # Build interface parameters
 
@@ -414,9 +513,31 @@ sub buildConfig
         $interface->{'param'}{'interface-name'} =
             $interface->{$data->{'nameref'}{'ifReferenceName'}};
 
+        $interface->{'param'}{'node-display-name'} =
+            $interface->{$data->{'nameref'}{'ifReferenceName'}};
+
         $interface->{'param'}{'interface-nick'} =
             $interface->{$data->{'nameref'}{'ifNick'}};
 
+        if( not defined( $interface->{$data->{'nameref'}{'ifNodeidPrefix'}} ) )
+        {
+            $interface->{$data->{'nameref'}{'ifNodeidPrefix'}} =
+                'if//%nodeid-device%//';
+        }
+        
+        if( not defined( $interface->{$data->{'nameref'}{'ifNodeid'}} ) )
+        {
+            $interface->{$data->{'nameref'}{'ifNodeid'}} =
+                $interface->{$data->{'nameref'}{'ifReferenceName'}};
+        }
+
+        # A per-interface value which is used by leafs in IF-MIB templates
+        $interface->{'param'}{'nodeid-interface'} =
+            $interface->{$data->{'nameref'}{'ifNodeidPrefix'}} .
+            $interface->{$data->{'nameref'}{'ifNodeid'}};
+        
+        $interface->{'param'}{'nodeid'} = '%nodeid-interface%';        
+
         if( defined $data->{'nameref'}{'ifComment'} and
             not defined( $interface->{'param'}{'comment'} ) and
             length( $interface->{$data->{'nameref'}{'ifComment'}} ) > 0 )
@@ -431,6 +552,35 @@ sub buildConfig
 
         $interface->{'param'}{'devdiscover-nodetype'} =
             'RFC2863_IF_MIB::interface';
+
+        if( $bandwidthUsageConfigured )
+        {
+            my $subtreeName =
+                $interface->{$data->{'nameref'}{'ifSubtreeName'}};
+            
+            if( defined( $bandwidthLimits{$subtreeName} ) )
+            {
+                $interface->{'param'}{'bandwidth-limit-in'} =
+                    $bandwidthLimits{$subtreeName}{'In'};
+                $interface->{'param'}{'bandwidth-limit-out'} =
+                    $bandwidthLimits{$subtreeName}{'Out'};
+            }
+
+            # Bandwidth usage parameters may be added by some other modules
+
+            if( defined($interface->{'param'}{'bandwidth-limit-in'}) and
+                defined($interface->{'param'}{'bandwidth-limit-out'}) and
+                ( $interface->{'hasOctets'} or $interface->{'hasHCOctets'} ) )
+            {
+                $bandwidthUsageActive = 1;
+                $interface->{'hasBandwidthUsage'} = 1;
+            }
+            else
+            {
+                delete $interface->{'param'}{'bandwidth-limit-in'};
+                delete $interface->{'param'}{'bandwidth-limit-out'};
+            }
+        }
     }
 
     if( $nInterfaces == 0 )
@@ -471,19 +621,6 @@ sub buildConfig
         }
     }
 
-    # Bandwidth usage
-    my %bandwidthLimits;
-    if( $devdetails->param('RFC2863_IF_MIB::bandwidth-usage') eq 'yes' )
-    {
-        my $limits = $devdetails->param('RFC2863_IF_MIB::bandwidth-limits');
-        foreach my $intfLimit ( split( /\s*;\s*/, $limits ) )
-        {
-            my( $intf, $limitIn, $limitOut ) = split( /\s*:\s*/, $intfLimit );
-            $bandwidthLimits{$intf}{'In'} = $limitIn;
-            $bandwidthLimits{$intf}{'Out'} = $limitOut;
-        }
-    }
-
     # tokenset member interfaces of the form
     # Format: tset:intf,intf; tokenset:intf,intf;
     # Format for global parameter:
@@ -638,7 +775,7 @@ sub buildConfig
         $subtreeParams->{'comment'} = $subtreeComment;
     }
 
-    if( $devdetails->param('RFC2863_IF_MIB::bandwidth-usage') eq 'yes' )
+    if( $bandwidthUsageActive )
     {
         $subtreeParams->{'overview-shortcuts'} = 'traffic,errors,bandwidth';
     }
@@ -700,6 +837,10 @@ sub buildConfig
 
         if( $interface->{'hasOctets'} or $interface->{'hasHCOctets'} )
         {
+            $interface->{'hasChild'}{'Bytes_In'} = 1;
+            $interface->{'hasChild'}{'Bytes_Out'} = 1;
+            $interface->{'hasChild'}{'InOut_bps'} = 1;            
+
             foreach my $dir ( 'In', 'Out' )
             {
                 if( defined( $interface->{'selectorActions'}->
@@ -732,50 +873,76 @@ sub buildConfig
             {
                 push( @templates, '::holt-winters-defaults' );
             }
+
+            if( defined( $interface->{'selectorActions'}{'NotifyPolicy'} ) )
+            {
+                $interface->{'param'}{'notify-policy'} =
+                    $interface->{'selectorActions'}{'NotifyPolicy'};
+            }
         }
 
         if( not $interface->{'selectorActions'}{'NoPacketCounters'} )
         {
+            my $has_someting = 0;
             if( $interface->{'hasHCUcastPkts'} )
             {
                 push( @templates, 'RFC2863_IF_MIB::ifxtable-hcucast-packets' );
+                $has_someting = 1;
             }
             elsif( $interface->{'hasUcastPkts'} )
             {
                 push( @templates, 'RFC2863_IF_MIB::iftable-ucast-packets' );
+                $has_someting = 1;
+            }
+
+            if( $has_someting )
+            {
+                $interface->{'hasChild'}{'Packets_In'} = 1;            
+                $interface->{'hasChild'}{'Packets_Out'} = 1;            
             }
         }
 
-        if( not $interface->{'selectorActions'}{'NoErrorCounters'} )
+        if( not $interface->{'selectorActions'}{'NoDiscardCounters'} )
         {
             if( $interface->{'hasInDiscards'} )
             {
                 push( @templates, 'RFC2863_IF_MIB::iftable-discards-in' );
-                if( defined( $interface->{'selectorActions'}->{
-                    'InErrorsMonitor'} ) )
+                $interface->{'hasChild'}{'Discards_In'} = 1;            
+
+                if( defined
+                    ($interface->{'selectorActions'}->{'InDiscardsMonitor'}) )
                 {
                     $interface->{'childCustomizations'}->{
                         'Discards_In'}->{'monitor'} =
-                            $interface->{'selectorActions'}{'InErrorsMonitor'};
+                            $interface->{'selectorActions'}{
+                                'InDiscardsMonitor'};
                 }
             }
-
+            
             if( $interface->{'hasOutDiscards'} )
             {
                 push( @templates, 'RFC2863_IF_MIB::iftable-discards-out' );
+                $interface->{'hasChild'}{'Discards_Out'} = 1;
+                
                 if( defined( $interface->{'selectorActions'}->{
-                    'OutErrorsMonitor'} ) )
+                    'OutDiscardsMonitor'} ) )
                 {
                     $interface->{'childCustomizations'}->{
                         'Discards_Out'}->{'monitor'} =
                             $interface->{'selectorActions'}{
-                                'OutErrorsMonitor'};
+                                'OutDiscardsMonitor'};
                 }
             }
+        }
+        
 
+        if( not $interface->{'selectorActions'}{'NoErrorCounters'} )
+        {
             if( $interface->{'hasInErrors'} )
             {
                 push( @templates, 'RFC2863_IF_MIB::iftable-errors-in' );
+                $interface->{'hasChild'}{'Errors_In'} = 1;            
+
                 if( defined( $interface->{'selectorActions'}->{
                     'InErrorsMonitor'} ) )
                 {
@@ -788,6 +955,8 @@ sub buildConfig
             if( $interface->{'hasOutErrors'} )
             {
                 push( @templates, 'RFC2863_IF_MIB::iftable-errors-out' );
+                $interface->{'hasChild'}{'Errors_Out'} = 1;            
+
                 if( defined( $interface->{'selectorActions'}->{
                     'OutErrorsMonitor'} ) )
                 {
@@ -820,24 +989,10 @@ sub buildConfig
             }
         }
 
-        if( $devdetails->param('RFC2863_IF_MIB::bandwidth-usage') eq 'yes' )
+        if( $bandwidthUsageActive and $interface->{'hasBandwidthUsage'} )
         {
-            if( defined( $bandwidthLimits{$subtreeName} ) )
-            {
-                $interface->{'param'}{'bandwidth-limit-in'} =
-                    $bandwidthLimits{$subtreeName}{'In'};
-                $interface->{'param'}{'bandwidth-limit-out'} =
-                    $bandwidthLimits{$subtreeName}{'Out'};
-            }
-
-            # We accept that parameters may be added by some other ways
-
-            if( defined( $interface->{'param'}{'bandwidth-limit-in'} ) and
-                defined( $interface->{'param'}{'bandwidth-limit-out'} ) )
-            {
-                push( @templates,
-                      'RFC2863_IF_MIB::interface-bandwidth-usage' );
-            }
+            push( @templates,
+                  'RFC2863_IF_MIB::interface-bandwidth-usage' );
         }
 
         if( ref( $interface->{'templates'} ) )
@@ -850,6 +1005,13 @@ sub buildConfig
         
         # Add subtree only if there are template references
 
+        if( defined( $interface->{'selectorActions'}{'AddTemplates'} ) )
+        {
+            push( @templates,
+                  split('\s*,\s*',
+                        $interface->{'selectorActions'}{'AddTemplates'}) );
+        }        
+            
         if( scalar( @templates ) > 0 )
         {
             # process interface-level parameters to copy
@@ -893,9 +1055,12 @@ sub buildConfig
                             $params->{'ext-service-trees'} =
                                 $extStorageTrees{$serviceid};
                         }
-                        
-                        $interface->{'childCustomizations'}->{
-                            'Bytes_' . $dir} = $params;
+
+                        foreach my $param ( keys %{$params} )
+                        {
+                            $interface->{'childCustomizations'}->{
+                                'Bytes_' . $dir}{$param} = $params->{$param};
+                        }
                     }
                 }
             }
@@ -909,9 +1074,13 @@ sub buildConfig
                 foreach my $childName
                     ( sort keys %{$interface->{'childCustomizations'}} )
                 {
-                    $cb->addLeaf
-                        ( $intfNode, $childName,
-                          $interface->{'childCustomizations'}->{$childName} );
+                    if( $interface->{'hasChild'}{$childName} )
+                    {
+                        $cb->addLeaf
+                            ( $intfNode, $childName,
+                              $interface->{'childCustomizations'}->{
+                                  $childName} );
+                    }
                 }
             }
 
@@ -1073,13 +1242,44 @@ sub addInterfaceFilter
 }
 
 
+# check if a given reference returns unique names and no empty values
+
+sub validateReference
+{
+    my $devdetails = shift;
+    my $nameref = shift;
+
+    my $data = $devdetails->data();
+    my %seen;
+
+    foreach my $ifIndex ( sort {$a<=>$b} keys %{$data->{'interfaces'}} )
+    {
+        my $interface = $data->{'interfaces'}{$ifIndex};
+
+        my $entry = $interface->{$nameref};
+        if( not defined($entry) or length($entry) == 0 )
+        {
+            Debug($nameref . ' contains empty entries');
+            return 0;
+        }
+        if( $seen{$entry} )
+        {
+            Debug($nameref . ' contains duplicate entries');
+            return 0;
+        }
+        $seen{$entry} = 1;
+    }
+    return 1;
+}
+
+
 sub uniqueEntries
 {
     my $devdetails = shift;
     my $nameref = shift;
 
     my $data = $devdetails->data();
-    my %count = ();
+    my %count;
 
     foreach my $ifIndex ( sort {$a<=>$b} keys %{$data->{'interfaces'}} )
     {
@@ -1238,16 +1438,21 @@ sub getSelectorObjectName
 our %knownSelectorActions =
     ( 'InBytesMonitor'    => 'RFC2863_IF_MIB',
       'OutBytesMonitor'   => 'RFC2863_IF_MIB',
+      'InDiscardsMonitor'  => 'RFC2863_IF_MIB',
+      'OutDiscardsMonitor' => 'RFC2863_IF_MIB',
       'InErrorsMonitor'   => 'RFC2863_IF_MIB',
       'OutErrorsMonitor'  => 'RFC2863_IF_MIB',
+      'NotifyPolicy'      => 'RFC2863_IF_MIB',
       'HoltWinters'       => 'RFC2863_IF_MIB',
       'NoPacketCounters'  => 'RFC2863_IF_MIB',
+      'NoDiscardCounters' => 'RFC2863_IF_MIB',
       'NoErrorCounters'   => 'RFC2863_IF_MIB',
       'RemoveInterface'   => 'RFC2863_IF_MIB',
       'TokensetMember'    => 'RFC2863_IF_MIB',
       'Parameters'        => 'RFC2863_IF_MIB',
       'InBytesParameters' => 'RFC2863_IF_MIB',
-      'OutBytesParameters' => 'RFC2863_IF_MIB',);
+      'OutBytesParameters' => 'RFC2863_IF_MIB',
+      'AddTemplates'       => 'RFC2863_IF_MIB' );
 
                             
 sub applySelectorAction
