@@ -17,7 +17,7 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Log.pm,v 1.4 2006/07/20 14:25:56 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 # 2002/06/25 11:35:00  ssinyagin
@@ -48,6 +48,13 @@ my %logLevel = (
 
 my $currentLogLevel = $logLevel{'info'};
 
+our $TID = 0;
+sub setTID
+{
+    $TID = shift;
+}
+
+
 sub Log
 {
     my( $level, @msg ) = @_;    
@@ -56,6 +63,10 @@ sub Log
     
     if( $level <= $currentLogLevel )
     {
+        if( $currentLogLevel >= 9 )
+        {
+            unshift(@msg, $$ . '.' . $TID . ' ');
+        }
         my $severity = ( $level <= $logLevel{'warn'} ) ? '*' : ' ';
         printf STDERR ( "[%s%s] %s\n",
                         timeStr( time() ), $severity, join( '', @msg ) );
@@ -84,15 +95,9 @@ sub Verbose
     Log( 'verbose', @_ );
 }
 
-our $TID = 0;
-sub setTID
-{
-    $TID = shift;
-}
-
 sub Debug
 {
-    Log( 'debug', $$ . '.' . $TID . ' ', join('|', @_) );
+    Log( 'debug', join('|', @_) );
 }
 
 
