@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: Monitor.pm,v 1.18 2008/09/25 21:24:45 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Monitor;
@@ -114,6 +114,8 @@ sub run
             undef $obj;
         }
     }
+
+    $self->cleanupExpired();
     
     undef $self->{'db_alarms'};
 }
@@ -242,7 +244,9 @@ sub setAlarm
     my $alarm = $obj->{'alarm'};
     my $timestamp = $obj->{'timestamp'};
 
-    my $prev_values = $self->{'db_alarms'}->get( $token.':'.$mname );
+    my $key = $mname . ':' . $config_tree->path($token);
+    
+    my $prev_values = $self->{'db_alarms'}->get( $key );
     my ($t_set, $t_expires, $prev_status, $t_last_change);
     if( defined($prev_values) )
     {
@@ -254,12 +258,13 @@ sub setAlarm
 
     my $event;
 
+    $t_last_change = time();
+    
     if( $alarm )
     {
         if( not $prev_status )
         {
             $t_set = $timestamp;
-            $t_last_change = time();
             $event = 'set';
         }
         else
@@ -271,7 +276,6 @@ sub setAlarm
     {
         if( $prev_status )
         {
-            $t_last_change = time();
             $t_expires = $t_last_change +
                 $config_tree->getParam($mname, 'expires');
             $event = 'clear';
@@ -280,7 +284,7 @@ sub setAlarm
         {
             if( defined($t_expires) and time() > $t_expires )
             {
-                $self->{'db_alarms'}->del( $token.':'.$mname );
+                $self->{'db_alarms'}->del( $key );
                 $event = 'forget';
             }
         }
@@ -316,7 +320,7 @@ sub setAlarm
 
         if( $event ne 'forget' )
         {
-            $self->{'db_alarms'}->put( $token.':'.$mname,
+            $self->{'db_alarms'}->put( $key,
                                        join(':', ($t_set,
                                                   $t_expires,
                                                   ($alarm ? 1:0),
@@ -326,6 +330,41 @@ sub setAlarm
 }
 
 
+# If an alarm is no longer in ConfigTree, it is not cleaned by setAlarm.
+# We clean them up explicitly after they expire
+
+sub cleanupExpired
+{
+    my $self = shift;
+
+    &Torrus::DB::checkInterrupted();
+    
+    my $cursor = $self->{'db_alarms'}->cursor(-Write => 1);
+    while( my ($key, $timers) = $self->{'db_alarms'}->next($cursor) )
+    {
+        my ($t_set, $t_expires, $prev_status, $t_last_change) =
+            split(':', $timers);
+        
+        if( $t_last_change and
+            time() > ( $t_last_change + $Torrus::Monitor::alarmTimeout ) and
+            ( (not $t_expires) or (time() > $t_expires) ) )
+        {            
+            my ($mname, $path) = split(':', $key);
+            
+            Info('Cleaned up an orphaned alarm: monitor=' . $mname .
+                 ', path=' . $path);
+            $self->{'db_alarms'}->c_del( $cursor );            
+        }
+    }
+    $self->{'db_alarms'}->c_close($cursor);
+    
+    &Torrus::DB::checkInterrupted();
+}
+    
+
+
+    
+
 sub run_event_tset
 {
     my $self = shift;
@@ -342,7 +381,7 @@ sub run_event_tset
 
         if( $event eq 'set' )
         {
-            $config_tree->tsetAddMember($tset, $token);
+            $config_tree->tsetAddMember($tset, $token, 'monitor');
         }
         else
         {
@@ -524,7 +563,7 @@ sub beforeRun
             sleep( $self->{'delay'} );
         }
 
-        Info("Rebuilding monitor cache and clearing alarms");
+        Info("Rebuilding monitor cache");
         Debug("Config TS: $actual_ts, Monitor TS: $known_ts");
 
         undef $data->{'targets'};
@@ -540,13 +579,6 @@ sub beforeRun
         $data->{'db_tokens'}->closeNow();
         undef $data->{'db_tokens'};
 
-        # Remove the alarms state information
-        $self->{'db_alarms'} = new Torrus::DB('monitor_alarms',
-                                              -Subdir => $tree,
-                                              -WriteAccess => 1);
-        $self->{'db_alarms'}->trunc();
-        
-        undef $self->{'db_alarms'};
         # Set the timestamp
         &Torrus::TimeStamp::setNow($tree . ':monitor_cache');
     }
@@ -572,6 +604,7 @@ sub beforeRun
             $data->{'mlist'}{$token} = [];
             push( @{$data->{'mlist'}{$token}}, split(',', $mlist) );
         }
+        $data->{'db_tokens'}->c_close($cursor);
         undef $cursor;
         $data->{'db_tokens'}->closeNow();
         undef $data->{'db_tokens'};
