@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: AdmInfo.pm,v 1.3 2007/06/16 16:54:09 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer::AdmInfo;
@@ -76,7 +76,8 @@ my %leaf_params =
      'rrd-scaling-base'         => undef,
      'graph-logarithmic'        => undef,
      'graph-rigid-boundaries'   => undef,
-     'graph-ignore-decorations' => undef);
+     'graph-ignore-decorations' => undef,
+     'nodeid'                   => undef);
 
 
 my %param_categories =
@@ -100,6 +101,7 @@ my %param_categories =
      'graph-logarithmic'                 => 'Display',
      'graph-rigid-boundaries'            => 'Display',
      'leaf-type'                         => 'Common Parameters',
+     'nodeid'                            => 'Common Parameters',
      'rpn-expr'                          => 'RRD CDEF Paramters',
      'rrd-cf'                            => 'RRD',
      'rrd-create-dstype'                 => 'RRD',
