@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: HTML.pm,v 1.14 2009/02/16 11:48:51 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer::HTML;
@@ -45,6 +45,8 @@ sub render_html
 
     my $tmplfile = $config_tree->getParam($view, 'html-template');
 
+    my $expires = $config_tree->getParam($view, 'expires');
+    
     # Create the Template Toolkit processor once, and reuse
     # it in subsequent render() calls
 
@@ -59,6 +61,7 @@ sub render_html
         'treeName'   => $config_tree->treeName(),
         'token'      => $token,
         'view'       => $view,
+        'expires'    => $expires,
         'path'       => sub { return $config_tree->path($_[0]); },
         'pathToken'  => sub { return $config_tree->token($_[0]); },
         'nodeExists' => sub { return $config_tree->nodeExists($_[0]); },
@@ -67,12 +70,11 @@ sub render_html
         'isAlias'    => sub { return $config_tree->isAlias($_[0]); },
         'sortTokens' => sub { return $self->sortTokens($config_tree,
                                                        $_[0]); },
-        'nodeName'   => sub { return $config_tree->nodeName($_[0]); },
+        'nodeName'   => sub { return $self->nodeName($config_tree, $_[0]); },
         'parent'     => sub { return $config_tree->getParent($_[0]); },
         'nodeParam'  => sub { return $config_tree->getNodeParam(@_); },
         'param'      => sub { return $config_tree->getParam(@_); },
-        'url'        => sub { return $self->makeURL($config_tree, 0, @_); },
-        'pathUrl'    => sub { return $self->makeURL($config_tree, 1, @_); },
+        'url'        => sub { return $self->makeURL($config_tree, @_); },
         'clearVar'   => sub { delete $self->{'options'}{'variables'}{$_[0]};
                               return undef;},
         'plainURL'   => $Torrus::Renderer::plainURL,
@@ -94,7 +96,7 @@ sub render_html
             $config_tree->treeName()}{'info'}; },
         'version'    => $Torrus::Global::version,
         'xmlnorm'    => \&Torrus::Renderer::xmlnormalize,
-        'userAuth'   => $Torrus::ApacheHandler::authorizeUsers,
+        'userAuth'   => $Torrus::CGI::authorizeUsers,
         'uid'        => $self->{'options'}->{'uid'},
         'userAttr'   => sub { return $self->userAttribute( $_[0] ) },
         'mayDisplayAdmInfo' => sub {
@@ -139,8 +141,23 @@ sub render_html
         return undef;
     }
 
-    return ($config_tree->getParam($view, 'expires')+time(),
-            'text/html; charset=UTF-8');
+    return ($expires+time(), 'text/html; charset=UTF-8');
+}
+
+
+sub nodeName
+{
+    my $self = shift;
+    my $config_tree = shift;
+    my $token = shift;
+
+    my $n = $config_tree->getNodeParam($token, 'node-display-name', 1);
+    if( defined( $n ) and length( $n ) > 0 )
+    {
+        return $n;
+    }
+    
+    return $config_tree->nodeName($config_tree->path($token));
 }
 
 
@@ -179,25 +196,39 @@ sub sortTokens
 }
 
 
+# compose an URL for a node.
+# Link is done with nodeid if available, or with path
+
 sub makeURL
 {
     my $self = shift;
     my $config_tree = shift;
-    my $pathref = shift;
     my $token = shift;
     my $view = shift;
     my @add_vars = @_;
 
     my $ret = $Torrus::Renderer::rendererURL . '/' . $config_tree->treeName();
 
-    if( $pathref )
+    # Try using nodeid whenever it's available
+
+    if( $config_tree->isTset($token) )
     {
-        my $path = $config_tree->path($token);
-        $ret .= '?path=' . uri_escape($path);
+        $ret .= '?token=' . $token;
     }
     else
     {
-        $ret .= '?token=' . uri_escape($token);
+        my $nodeid = $config_tree->getNodeParam($token, 'nodeid', 1);
+        if( defined( $nodeid ) )
+        {
+            $ret .= '?nodeid=' .
+                uri_escape($nodeid, $Torrus::Renderer::uriEscapeExceptions);
+        }
+        else
+        {
+            $ret .= '?path=' .
+                uri_escape($config_tree->path($token),
+                           $Torrus::Renderer::uriEscapeExceptions);
+        }
     }
 
     if( $view )
@@ -206,18 +237,15 @@ sub makeURL
     }
 
     my %vars = ();
-    if( scalar( @add_vars ) )
+    # This could be array or a reference to array
+    my $add_vars_size = scalar( @add_vars );
+    if( $add_vars_size == 1 and ref( $add_vars[0] ) )
     {
-        # This could be array or a reference to array
-        my $rvars;
-        if( ref( $add_vars[0] ) )
-        {
-            %vars = @{$add_vars[0]};
-        }
-        else
-        {
-            %vars = @add_vars;
-        }
+        %vars = @{$add_vars[0]};
+    }
+    elsif( $add_vars_size > 0 and ($add_vars_size % 2 == 0) )
+    {
+        %vars = @add_vars;
     }
 
     if( ref( $self->{'options'}->{'variables'} ) )
@@ -236,7 +264,9 @@ sub makeURL
     {
         if( $vars{$name} ne '' )
         {
-            $ret .= '&amp;'.$name.'='.uri_escape( $vars{$name} );
+            $ret .= '&amp;' . $name . '=' .
+                uri_escape( $vars{$name},
+                            $Torrus::Renderer::uriEscapeExceptions );
         }
     }
 
@@ -250,26 +280,25 @@ sub makeSplitURLs
     my $token = shift;
     my $view = shift;
 
-    my $path = $config_tree->path($token);
-
     my $ret = '';
-    my $currpath = '';
-    foreach my $node ($config_tree->splitPath($path))
+    while( defined( $token ) )
     {
-        $currpath .= $node;
-        if( not defined(my $currtoken = $config_tree->token($currpath)) )
-        {
-            Error("Cannot find token for $currpath");
-        }
-        else
-        {
-            $ret .= '<SPAN CLASS="PathElement">';
-            $ret .= sprintf('<A HREF="%s">%s</A>',
-                            $self->makeURL($config_tree, 0, $currtoken, $view),
-                            $node);
-            $ret .= "</SPAN>\n";
-        }
+        my $path = $config_tree->path($token);
+        
+        my $str = '<SPAN CLASS="PathElement">';
+        $str .=
+            sprintf('<A HREF="%s">%s%s</A>',
+                    $self->makeURL($config_tree, $token, $view),
+                    $config_tree->nodeName($path),
+                    ( $config_tree->isSubtree($token) and
+                      $path ne '/') ? '/':'' );
+        $str .= "</SPAN>\n";
+        
+        $ret = $str . $ret;
+                
+        $token = $config_tree->getParent( $token );
     }
+    
     return $ret;
 }
 
@@ -318,7 +347,7 @@ sub scale
                    "n", # 10e-9  Nano
                    "u", # 10e-6  Micro
                    "m", # 10e-3  Milli
-                   " ", # Base
+                   "", # Base
                    "k", # 10e3   Kilo
                    "M", # 10e6   Mega
                    "G", # 10e9   Giga
@@ -328,7 +357,7 @@ sub scale
 
     my $symbcenter = 6;
     my $digits = (0 == $value)? 0 : floor(log(abs($value))/log(1000));
-    return sprintf( $fmt . " %s", $value/pow(1000, $digits),
+    return sprintf( $fmt . "%s", $value/pow(1000, $digits),
                     $symbols[ $symbcenter+$digits ] );
 }
 
@@ -442,7 +471,7 @@ sub may_display_reports
 
     if( $Torrus::Renderer::displayReports )
     {
-        if( not $Torrus::ApacheHandler::authorizeUsers )
+        if( not $Torrus::CGI::authorizeUsers )
         {
             return 1;
         }
