@@ -1,4 +1,4 @@
-#  Copyright (C) 2002  Stanislav Sinyagin
+#  Copyright (C) 2002-2011  Stanislav Sinyagin
 #
 #  This program is free software; you can redistribute it and/or modify
 #  it under the terms of the GNU General Public License as published by
@@ -14,7 +14,7 @@
 #  along with this program; if not, write to the Free Software
 #  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 
-# $Id: RRDtool.pm,v 1.9 2008/08/07 18:42:12 ssinyagin Exp $
+# $Id$
 # Stanislav Sinyagin <ssinyagin@yahoo.com>
 
 package Torrus::Renderer::RRDtool;
@@ -34,9 +34,16 @@ my %rrd_graph_opts =
      'start'  => '--start',
      'end'    => '--end',
      'width'  => '--width',
-     'height' => '--height'
+     'height' => '--height',
+     'imgformat' => '--imgformat', 
      );
 
+my %mime_type =
+    ('PNG' => 'image/png',
+     'SVG' => 'image/svg+xml',
+     'EPS' => 'application/postscript',
+     'PDF' => 'application/pdf');
+
 my @arg_arrays = qw(opts defs bg hwtick hrule hwline line fg);
 
 
@@ -63,7 +70,7 @@ sub render_rrgraph
 
     push( @{$obj->{'args'}{'opts'}},
           $self->rrd_make_opts( $config_tree, $token, $view,
-                                \%rrd_graph_opts, ) );
+                                \%rrd_graph_opts, $obj ) );
 
     push( @{$obj->{'args'}{'opts'}},
           $self->rrd_make_graph_opts( $config_tree, $token, $view ) );
@@ -82,9 +89,11 @@ sub render_rrgraph
         # At the moment, we call the DEF as 'A'. Could change in the future
         if( $leaftype eq 'rrd-def' )
         {
-            push( @{$obj->{'args'}{'defs'}},
-                  $self->rrd_make_def( $config_tree, $token,
-                                       $obj->{'dname'} ) );
+            my $defstring =
+                $self->rrd_make_def( $config_tree, $token, $obj->{'dname'} );
+            return(undef) unless defined($defstring);
+            
+            push( @{$obj->{'args'}{'defs'}}, $defstring );
 
             if( $self->rrd_check_hw( $config_tree, $token, $view ) )
             {
@@ -108,6 +117,8 @@ sub render_rrgraph
         $self->rrd_make_graphline( $config_tree, $token, $view, $obj );
     }
 
+    return(undef) if $obj->{'error'};
+    
     $self->rrd_make_hrules( $config_tree, $token, $view, $obj );
     if( not $Torrus::Renderer::ignoreDecorations )
     {
@@ -135,7 +146,13 @@ sub render_rrgraph
         return undef;
     }
 
-    return( $config_tree->getParam($view, 'expires')+time(), 'image/png' );
+    my $mimetype = $obj->{'mimetype'};
+    if( not defined($mimetype) )
+    {
+        $mimetype = 'image/png';
+    }
+            
+    return( $config_tree->getParam($view, 'expires')+time(), $mimetype );
 }
 
 
@@ -184,14 +201,15 @@ sub render_rrprint
     my $dname = 'A';
     if( $leaftype eq 'rrd-def' )
     {
-        push( @arg_defs,
-              $self->rrd_make_def( $config_tree, $token, $dname ) );
+        my $defstring = $self->rrd_make_def( $config_tree, $token, $dname );
+        return(undef) unless defined($defstring);
+        push( @arg_defs, $defstring );
     }
     elsif( $leaftype eq 'rrd-cdef' )
     {
         my $expr = $config_tree->getNodeParam($token, 'rpn-expr');
         push( @arg_defs,
-              $self->rrd_make_cdef($config_tree, $token, $dname, $expr) );
+              $self->rrd_make_cdef($config_tree, $token, $dname, $expr));
     }
     else
     {
@@ -259,12 +277,21 @@ sub rrd_make_multigraph
         $dsOrder{$dname} = defined( $order ) ? $order : 100;
     }
 
+    my $disable_legend = $config_tree->getParam($view, 'disable-legend');    
+    $disable_legend =
+        (defined($disable_legend) and $disable_legend eq 'yes') ? 1:0;
+    
     # make DEFs and Line instructions
 
-    my $do_gprint = $self->rrd_if_gprint( $config_tree, $token );
-    if( $do_gprint )
+    my $do_gprint = 0;
+
+    if( not $disable_legend )
     {
-        $self->rrd_make_gprint_header( $config_tree, $token, $view, $obj );
+        $do_gprint = $self->rrd_if_gprint( $config_tree, $token );
+        if( $do_gprint )
+        {
+            $self->rrd_make_gprint_header( $config_tree, $token, $view, $obj );
+        }
     }
 
     foreach my $dname ( sort {$dsOrder{$a} <=> $dsOrder{$b}} @dsNames )
@@ -294,9 +321,15 @@ sub rrd_make_multigraph
         if( $dograph or $gprint_this )
         {
             my $expr = $config_tree->getNodeParam($token, 'ds-expr-'.$dname);
-            push( @{$obj->{'args'}{'defs'}},
-                  $self->rrd_make_cdef($config_tree, $token, $dname, $expr) );
-
+            my @cdefs =
+                $self->rrd_make_cdef($config_tree, $token, $dname, $expr);
+            if( not scalar(@cdefs) )
+            {
+                $obj->{'error'} = 1;
+                next;
+            }
+            
+            push( @{$obj->{'args'}{'defs'}}, @cdefs );
             
             $legend =
                 $config_tree->getNodeParam($token, 'graph-legend-'.$dname);
@@ -337,10 +370,29 @@ sub rrd_make_multigraph
                 $self->mkcolor( $config_tree->getNodeParam
                                 ($token, 'line-color-'.$dname) );
             
+            my $alpha =
+                $config_tree->getNodeParam($token, 'line-alpha-'.$dname);
+            if( defined( $alpha ) )
+            {
+                $linecolor .= $alpha;
+            }
+
+            my $stack =
+                $config_tree->getNodeParam($token, 'line-stack-'.$dname);
+            if( defined( $stack ) and $stack eq 'yes' )
+            {
+                $stack = ':STACK';
+            }
+            else
+            {
+                $stack = '';
+            }
+                
             push( @{$obj->{'args'}{'line'}},
-                  sprintf( '%s:%s%s%s', $linestyle, $dname,
+                  sprintf( '%s:%s%s%s%s', $linestyle, $dname,
                            $linecolor,
-                           length($legend) > 0 ? ':'.$legend.'\l' : '' ) );
+                           length($legend) > 0 ? ':'.$legend.'\l' : '',
+                           $stack ) );
             
         }
     }
@@ -357,12 +409,13 @@ sub rrd_check_hw
 
     my $use_hw = 0;
     my $nodeHW = $config_tree->getNodeParam($token, 'rrd-hwpredict');
-    if( $nodeHW eq 'enabled' )
+    if( defined($nodeHW) and $nodeHW eq 'enabled' )
     {
         my $viewHW = $config_tree->getParam($view, 'rrd-hwpredict');
         my $varNoHW = $self->{'options'}->{'variables'}->{'NOHW'};
-
-        if( ($viewHW ne 'disabled') and (not $varNoHW) )
+        
+        if( (not defined($viewHW) or $viewHW ne 'disabled') and
+            (not $varNoHW) )
         {
             $use_hw = 1;
         }
@@ -381,12 +434,16 @@ sub rrd_make_holtwinters
 
     my $dname = $obj->{'dname'};
 
-    push( @{$obj->{'args'}{'defs'}},
-          $self->rrd_make_def( $config_tree, $token,
-                               $dname . 'pred', 'HWPREDICT' ) );
-    push( @{$obj->{'args'}{'defs'}},
-          $self->rrd_make_def( $config_tree, $token,
-                               $dname . 'dev', 'DEVPREDICT' ) );
+    my $defstring = $self->rrd_make_def( $config_tree, $token,
+                                         $dname . 'pred', 'HWPREDICT' );
+    return() unless defined($defstring);
+    push( @{$obj->{'args'}{'defs'}}, $defstring );
+
+    $defstring = $self->rrd_make_def( $config_tree, $token,
+                                      $dname . 'dev', 'DEVPREDICT' );
+    return() unless defined($defstring);
+    push( @{$obj->{'args'}{'defs'}}, $defstring );
+    
     # Upper boundary definition
     push( @{$obj->{'args'}{'defs'}},
           sprintf( 'CDEF:%supper=%spred,%sdev,2,*,+',
@@ -398,9 +455,10 @@ sub rrd_make_holtwinters
                    $dname, $dname, $dname  ) );
 
     # Failures definition
-    push( @{$obj->{'args'}{'defs'}},
-          $self->rrd_make_def( $config_tree, $token,
-                               $dname . 'fail', 'FAILURES' ) );
+    $defstring = $self->rrd_make_def( $config_tree, $token,
+                                      $dname . 'fail', 'FAILURES' );
+    return() unless defined($defstring);    
+    push( @{$obj->{'args'}{'defs'}}, $defstring );
 
     # Generate H-W Boundary Lines
 
@@ -441,16 +499,23 @@ sub rrd_make_graphline
     my $view = shift;
     my $obj = shift;
 
-    my $legend = $config_tree->getNodeParam($token, 'graph-legend');
-    if( defined( $legend ) )
+    my $legend;
+    
+    my $disable_legend = $config_tree->getParam($view, 'disable-legend');
+    if( not defined($disable_legend) or $disable_legend ne 'yes' )
     {
-        $legend =~ s/:/\\:/g;
+        $legend = $config_tree->getNodeParam($token, 'graph-legend');
+        if( defined( $legend ) )
+        {
+            $legend =~ s/:/\\:/g;
+        }
     }
-    else
+
+    if( not defined( $legend ) )
     {
         $legend = '';
     }
-
+    
     my $styleval = $config_tree->getNodeParam($token, 'line-style');
     if( not defined( $styleval ) or length( $styleval ) == 0 )
     {
@@ -532,9 +597,10 @@ sub rrd_make_decorations
     my $obj = shift;
 
     my $decorList = $config_tree->getParam($view, 'decorations');
+    my $ignore_decor =
+        $config_tree->getNodeParam($token, 'graph-ignore-decorations');
     if( defined( $decorList ) and
-        $config_tree->getNodeParam($token, 'graph-ignore-decorations')
-        ne 'yes' )
+        (not defined($ignore_decor) or $ignore_decor ne 'yes') )
     {
         my $decor = {};
         foreach my $decorName ( split(',', $decorList ) )
@@ -551,13 +617,20 @@ sub rrd_make_decorations
                                 getParam($view, 'dec-color-' . $decorName) );
             my $expr = $config_tree->
                 getParam($view, 'dec-expr-' . $decorName);
-
-            push( @{$decor->{$order}{'def'}},
-                  $self->rrd_make_cdef( $config_tree, $token, $decorName,
-                                        $obj->{'dname'} . ',POP,' . $expr ) );
-
-            $decor->{$order}{'line'} =
-                sprintf( '%s:%s%s', $style, $decorName, $color );
+            
+            my @cdefs =
+                $self->rrd_make_cdef( $config_tree, $token, $decorName,
+                                      $obj->{'dname'} . ',POP,' . $expr );
+            if( scalar(@cdefs) )
+            {
+                push( @{$decor->{$order}{'def'}}, @cdefs );
+                $decor->{$order}{'line'} =
+                    sprintf( '%s:%s%s', $style, $decorName, $color );
+            }
+            else
+            {
+                $obj->{'error'} = 1;
+            }
         }
 
         foreach my $order ( sort {$a<=>$b} keys %{$decor} )
@@ -580,6 +653,7 @@ sub rrd_make_opts
     my $token = shift;
     my $view = shift;
     my $opthash = shift;
+    my $obj = shift;
 
     my @args = ();
     foreach my $param ( keys %{$opthash} )
@@ -607,6 +681,20 @@ sub rrd_make_opts
                     $value = $now . $value;
                 }
             }
+            elsif( $param eq 'imgformat' )
+            {
+                if( not defined($mime_type{$value}) )
+                {
+                    Error('Unsupported value for imgformat: ' . $value);
+                    $value = 'PNG';
+                }
+
+                if( defined($obj) )
+                {
+                    $obj->{'mimetype'} = $mime_type{$value};
+                }
+            }
+            
             push( @args, $opthash->{$param}, $value );
         }
     }
@@ -634,51 +722,66 @@ sub rrd_make_graph_opts
     my $token = shift;
     my $view = shift;
 
-    my @args = ( '--imgformat', 'PNG' );
-
-    if( $config_tree->getNodeParam($token, 'graph-logarithmic') eq 'yes' )
+    my @args;
+    
+    my $graph_log = $config_tree->getNodeParam($token, 'graph-logarithmic');
+    if( defined($graph_log) and $graph_log eq 'yes' )
     {
         push( @args, '--logarithmic' );
     }
 
-    my $title = $config_tree->getNodeParam($token, 'graph-title');
-    if( not defined( $title ) or length( $title ) == 0 )
+    my $disable_title =
+        $config_tree->getParam($view, 'disable-title');
+    if( not defined( $disable_title ) or $disable_title ne 'yes' )
     {
-        $title = ' ';
+        my $title = $config_tree->getNodeParam($token, 'graph-title');
+        if( not defined( $title ) or length( $title ) == 0 )
+        {
+            $title = ' ';
+        }
+        push( @args, '--title', $title );
     }
-    push( @args, '--title', $title );
-    
-    my $vertical_label = $config_tree->getNodeParam($token, 'vertical-label');
-    if( not defined( $vertical_label ) or length( $vertical_label ) == 0 )
-    {
-        $vertical_label = ' ';
-    }
-    push( @args, '--vertical-label', $vertical_label );
 
-    if( $config_tree->getParam($view, 'ignore-limits') ne 'yes' )
+    my $disable_vlabel =
+        $config_tree->getParam($view, 'disable-vertical-label');
+    if( not defined( $disable_vlabel ) or $disable_vlabel ne 'yes' )
     {
-        if( $config_tree->getParam($view, 'ignore-lower-limit') ne 'yes' )
+        my $vertical_label =
+            $config_tree->getNodeParam($token, 'vertical-label');
+        if( defined( $vertical_label ) and length( $vertical_label ) > 0 )
+        {
+            push( @args, '--vertical-label', $vertical_label );
+        }
+    }
+
+    my $ignore_limits = $config_tree->getParam($view, 'ignore-limits');
+    if( not defined($ignore_limits) or $ignore_limits ne 'yes' )
+    {
+        my $ignore_lower = $config_tree->getParam($view, 'ignore-lower-limit');
+        if( not defined($ignore_lower) or $ignore_lower ne 'yes' )
         {
             my $limit =
                 $config_tree->getNodeParam($token, 'graph-lower-limit');
-            if( length( $limit ) > 0 )
+            if( defined($limit) and length( $limit ) > 0 )
             {
                 push( @args, '--lower-limit', $limit );
             }
         }
 
-        if( $config_tree->getParam($view, 'ignore-upper-limit') ne 'yes' )
+        my $ignore_upper = $config_tree->getParam($view, 'ignore-upper-limit');
+        if( not defined($ignore_upper) or $ignore_upper ne 'yes' )
         {
             my $limit =
                 $config_tree->getNodeParam($token, 'graph-upper-limit');
-            if( length( $limit ) > 0 )
+            if( defined($limit) and length( $limit ) > 0 )
             {
                 push( @args, '--upper-limit', $limit );
             }
         }
 
-        if( $config_tree->getNodeParam($token, 'graph-rigid-boundaries') eq
-            'yes' )
+        my $rigid_boundaries =
+            $config_tree->getNodeParam($token, 'graph-rigid-boundaries');
+        if( defined($rigid_boundaries) and $rigid_boundaries eq 'yes' )
         {
             push( @args, '--rigid' );
         }
@@ -738,7 +841,8 @@ sub rrd_make_cdef
     my $expr  = shift;
 
     my @args = ();
-
+    my $ok = 1;
+    
     # We will name the DEFs as $dname.sprintf('%.2d', $ds_couter++);
     my $ds_couter = 1;
 
@@ -765,13 +869,23 @@ sub rrd_make_cdef
             $config_tree->getRelative($token, $noderef) : $token;
 
         my $varname = $dname . sprintf('%.2d', $ds_couter++);
-        push( @args,
-              $self->rrd_make_def( $config_tree, $leaf, $varname, $cf ) );
+        my $defstring =
+            $self->rrd_make_def( $config_tree, $leaf, $varname, $cf );
+        if( not defined($defstring) )
+        {
+            $ok = 0;
+        }
+        else
+        {
+            push( @args, $defstring );
+        }
         return $varname;
     };
-
+    
     $expr = $rpn->translate( $expr, $callback );
+    return() unless $ok;
     push( @args, sprintf( 'CDEF:%s=%s', $dname, $expr ) );
+    
     return @args;
 }
 
